@@ -20,91 +20,106 @@
 
 #include "caf/config.hpp"
 
-#include "caf/sec.hpp"
-#include "caf/atom.hpp"
-#include "caf/send.hpp"
-#include "caf/skip.hpp"
-#include "caf/unit.hpp"
-#include "caf/term.hpp"
-#include "caf/actor.hpp"
-#include "caf/after.hpp"
-#include "caf/error.hpp"
-#include "caf/group.hpp"
-#include "caf/extend.hpp"
-#include "caf/logger.hpp"
-#include "caf/others.hpp"
-#include "caf/result.hpp"
-#include "caf/stream.hpp"
-#include "caf/message.hpp"
-#include "caf/node_id.hpp"
-#include "caf/behavior.hpp"
-#include "caf/defaults.hpp"
-#include "caf/duration.hpp"
-#include "caf/expected.hpp"
-#include "caf/exec_main.hpp"
-#include "caf/resumable.hpp"
-#include "caf/streambuf.hpp"
-#include "caf/to_string.hpp"
-#include "caf/actor_addr.hpp"
-#include "caf/actor_pool.hpp"
-#include "caf/attachable.hpp"
-#include "caf/message_id.hpp"
-#include "caf/replies_to.hpp"
-#include "caf/serializer.hpp"
-#include "caf/actor_clock.hpp"
-#include "caf/actor_proxy.hpp"
-#include "caf/exit_reason.hpp"
-#include "caf/local_actor.hpp"
-#include "caf/raise_error.hpp"
-#include "caf/ref_counted.hpp"
-#include "caf/stream_slot.hpp"
-#include "caf/thread_hook.hpp"
-#include "caf/typed_actor.hpp"
-#include "caf/actor_system.hpp"
-#include "caf/config_value.hpp"
-#include "caf/deserializer.hpp"
-#include "caf/scoped_actor.hpp"
-#include "caf/upstream_msg.hpp"
-#include "caf/actor_ostream.hpp"
-#include "caf/config_option.hpp"
-#include "caf/function_view.hpp"
-#include "caf/index_mapping.hpp"
-#include "caf/spawn_options.hpp"
 #include "caf/abstract_actor.hpp"
+#include "caf/abstract_channel.hpp"
+#include "caf/abstract_composable_behavior.hpp"
 #include "caf/abstract_group.hpp"
-#include "caf/blocking_actor.hpp"
-#include "caf/deep_to_string.hpp"
-#include "caf/execution_unit.hpp"
-#include "caf/memory_managed.hpp"
-#include "caf/stateful_actor.hpp"
-#include "caf/typed_behavior.hpp"
-#include "caf/proxy_registry.hpp"
-#include "caf/downstream_msg.hpp"
+#include "caf/actor.hpp"
+#include "caf/actor_addr.hpp"
+#include "caf/actor_clock.hpp"
+#include "caf/actor_ostream.hpp"
+#include "caf/actor_pool.hpp"
+#include "caf/actor_proxy.hpp"
+#include "caf/actor_system.hpp"
+#include "caf/actor_system_config.hpp"
+#include "caf/actor_traits.hpp"
+#include "caf/after.hpp"
+#include "caf/atom.hpp"
+#include "caf/attach_continuous_stream_source.hpp"
+#include "caf/attach_continuous_stream_stage.hpp"
+#include "caf/attach_stream_sink.hpp"
+#include "caf/attach_stream_source.hpp"
+#include "caf/attach_stream_stage.hpp"
+#include "caf/attachable.hpp"
+#include "caf/behavior.hpp"
 #include "caf/behavior_policy.hpp"
+#include "caf/binary_deserializer.hpp"
+#include "caf/binary_serializer.hpp"
+#include "caf/blocking_actor.hpp"
+#include "caf/composable_behavior.hpp"
+#include "caf/composed_behavior.hpp"
+#include "caf/config_option.hpp"
+#include "caf/config_option_adder.hpp"
+#include "caf/config_value.hpp"
+#include "caf/config_value_adaptor.hpp"
+#include "caf/config_value_adaptor_access.hpp"
+#include "caf/config_value_adaptor_field.hpp"
+#include "caf/config_value_field.hpp"
+#include "caf/config_value_object_access.hpp"
+#include "caf/deep_to_string.hpp"
+#include "caf/defaults.hpp"
+#include "caf/deserializer.hpp"
+#include "caf/detail/config_value_adaptor_field_impl.hpp"
+#include "caf/downstream_msg.hpp"
+#include "caf/duration.hpp"
+#include "caf/error.hpp"
+#include "caf/event_based_actor.hpp"
+#include "caf/exec_main.hpp"
+#include "caf/execution_unit.hpp"
+#include "caf/exit_reason.hpp"
+#include "caf/expected.hpp"
+#include "caf/extend.hpp"
+#include "caf/function_view.hpp"
+#include "caf/fused_downstream_manager.hpp"
+#include "caf/group.hpp"
+#include "caf/index_mapping.hpp"
+#include "caf/local_actor.hpp"
+#include "caf/logger.hpp"
+#include "caf/make_config_option.hpp"
+#include "caf/make_config_value_field.hpp"
+#include "caf/may_have_timeout.hpp"
+#include "caf/memory_managed.hpp"
+#include "caf/message.hpp"
 #include "caf/message_builder.hpp"
 #include "caf/message_handler.hpp"
-#include "caf/response_handle.hpp"
-#include "caf/system_messages.hpp"
-#include "caf/abstract_channel.hpp"
-#include "caf/may_have_timeout.hpp"
+#include "caf/message_id.hpp"
 #include "caf/message_priority.hpp"
-#include "caf/typed_actor_view.hpp"
-#include "caf/binary_serializer.hpp"
-#include "caf/composed_behavior.hpp"
-#include "caf/event_based_actor.hpp"
+#include "caf/node_id.hpp"
+#include "caf/others.hpp"
 #include "caf/primitive_variant.hpp"
-#include "caf/make_config_option.hpp"
-#include "caf/timeout_definition.hpp"
-#include "caf/actor_system_config.hpp"
-#include "caf/binary_deserializer.hpp"
-#include "caf/composable_behavior.hpp"
-#include "caf/config_option_adder.hpp"
-#include "caf/typed_actor_pointer.hpp"
+#include "caf/proxy_registry.hpp"
+#include "caf/raise_error.hpp"
+#include "caf/ref_counted.hpp"
+#include "caf/replies_to.hpp"
+#include "caf/response_handle.hpp"
+#include "caf/result.hpp"
+#include "caf/resumable.hpp"
+#include "caf/scoped_actor.hpp"
 #include "caf/scoped_execution_unit.hpp"
-#include "caf/typed_response_promise.hpp"
+#include "caf/sec.hpp"
+#include "caf/send.hpp"
+#include "caf/serializer.hpp"
+#include "caf/skip.hpp"
+#include "caf/spawn_options.hpp"
+#include "caf/stateful_actor.hpp"
+#include "caf/stream.hpp"
+#include "caf/stream_deserializer.hpp"
+#include "caf/stream_serializer.hpp"
+#include "caf/stream_slot.hpp"
+#include "caf/streambuf.hpp"
+#include "caf/system_messages.hpp"
+#include "caf/term.hpp"
+#include "caf/thread_hook.hpp"
+#include "caf/timeout_definition.hpp"
+#include "caf/to_string.hpp"
+#include "caf/typed_actor.hpp"
+#include "caf/typed_actor_pointer.hpp"
+#include "caf/typed_actor_view.hpp"
+#include "caf/typed_behavior.hpp"
 #include "caf/typed_event_based_actor.hpp"
-#include "caf/fused_downstream_manager.hpp"
-#include "caf/abstract_composable_behavior.hpp"
+#include "caf/typed_response_promise.hpp"
+#include "caf/unit.hpp"
+#include "caf/upstream_msg.hpp"
 
 #include "caf/decorator/sequencer.hpp"
 
@@ -232,10 +247,10 @@
 /// using result_atom = atom_constant<atom("result")>;
 ///
 /// // send a message to a1
-/// self->send(a1, hello_atom::value, "hello a1!");
+/// self->send(a1, hello_atom_v, "hello a1!");
 ///
 /// // send a message to a1, a2, and a3
-/// auto msg = make_message(compute_atom::value, 1, 2, 3);
+/// auto msg = make_message(compute_atom_v, 1, 2, 3);
 /// self->send(a1, msg);
 /// self->send(a2, msg);
 /// self->send(a3, msg);
@@ -254,7 +269,7 @@
 ///   },
 ///   [](compute_atom, int i0, int i1, int i2) {
 ///     // send our result back to the sender of this messages
-///     return make_message(result_atom::value, i0 + i1 + i2);
+///     return make_message(result_atom_v, i0 + i1 + i2);
 ///   }
 /// }
 /// ~~
@@ -274,11 +289,11 @@
 ///
 /// @section Atoms Atoms
 ///
-/// Atoms are a nice way to add semantic informations to a message.
-/// Assuming an actor wants to provide a "math sevice" for integers. It
-/// could provide operations such as addition, subtraction, etc.
-/// This operations all have two operands. Thus, the actor does not know
-/// what operation the sender of a message wanted by receiving just two integers.
+/// Atoms are a nice way to add semantic informations to a message. Assuming an
+/// actor wants to provide a "math sevice" for integers. It could provide
+/// operations such as addition, subtraction, etc. This operations all have two
+/// operands. Thus, the actor does not know what operation the sender of a
+/// message wanted by receiving just two integers.
 ///
 /// Example actor:
 /// ~~
@@ -356,14 +371,14 @@
 /// ~~
 /// scoped_actor self{...};
 ///
-/// self->delayed_send(self, std::chrono::seconds(1), poll_atom::value);
+/// self->delayed_send(self, std::chrono::seconds(1), poll_atom_v);
 /// bool running = true;
 /// self->receive_while([&](){ return running; }) (
 ///   // ...
 ///   [&](poll_atom) {
 ///     // ... poll something ...
 ///     // and do it again after 1sec
-///     self->delayed_send(self, std::chrono::seconds(1), poll_atom::value);
+///     self->delayed_send(self, std::chrono::seconds(1), poll_atom_v);
 ///   }
 /// );
 /// ~~
@@ -381,18 +396,15 @@
 ///
 /// A few examples:
 /// ~~
-/// // sends an std::string containing "hello actor!" to itself
+/// // sends a string containing "hello actor!" to itself
 /// send(self, "hello actor!");
 ///
 /// const char* cstring = "cstring";
-/// // sends an std::string containing "cstring" to itself
+/// // sends an string containing "cstring" to itself
 /// send(self, cstring);
 ///
-/// // sends an std::u16string containing the UTF16 string "hello unicode world!"
+/// // sends an u16string containing the UTF16 string "hello unicode world!"
 /// send(self, u"hello unicode world!");
-///
-/// // x has the type caf::tuple<std::string, std::string>
-/// auto x = make_message("hello", "tuple");
 /// ~~
 ///
 /// @defgroup ActorCreation Creating Actors
