@@ -23,7 +23,7 @@
 #include <condition_variable>
 
 #include "caf/actor_config.hpp"
-#include "caf/actor_marker.hpp"
+#include "caf/actor_traits.hpp"
 #include "caf/after.hpp"
 #include "caf/behavior.hpp"
 #include "caf/extend.hpp"
@@ -72,11 +72,14 @@ namespace caf {
 /// receive rather than a behavior-stack based message processing.
 /// @extends local_actor
 class blocking_actor
-    : public extend<local_actor, blocking_actor>::
-             with<mixin::requester,
-                  mixin::sender,
-                  mixin::subscriber>,
-      public dynamically_typed_actor_base {
+  // clang-format off
+  : public extend<local_actor, blocking_actor>::
+           with<mixin::requester,
+                mixin::sender,
+                mixin::subscriber>,
+    public dynamically_typed_actor_base,
+    public blocking_actor_base {
+  // clang-format on
 public:
   // -- nested and member types ------------------------------------------------
 
@@ -86,7 +89,7 @@ public:
   /// Stores asynchronous messages with default priority.
   using normal_queue = intrusive::drr_cached_queue<policy::normal_messages>;
 
-  /// Stores asynchronous messages with hifh priority.
+  /// Stores asynchronous messages with high priority.
   using urgent_queue = intrusive::drr_cached_queue<policy::urgent_messages>;
 
   /// Configures the FIFO inbox with two nested queues:
@@ -244,6 +247,8 @@ public:
 
   void enqueue(mailbox_element_ptr, execution_unit*) override;
 
+  mailbox_element* peek_at_next_mailbox_element() override;
+
   // -- overridden functions of local_actor ------------------------------------
 
   const char* name() const override;
@@ -338,7 +343,6 @@ public:
   /// Blocks this actor until all `xs...` have terminated.
   template <class... Ts>
   void wait_for(Ts&&... xs) {
-    using wait_for_atom = atom_constant<atom("waitFor")>;
     size_t expected = 0;
     size_t i = 0;
     size_t attach_results[] = {attach_functor(xs)...};
