@@ -73,24 +73,6 @@ public:
       std::u32string
     >;
 
-  /// List of builtin types for data processors as enum.
-  enum builtin {
-    i8_v,
-    u8_v,
-    i16_v,
-    u16_v,
-    i32_v,
-    u32_v,
-    i64_v,
-    u64_v,
-    float_v,
-    double_v,
-    ldouble_v,
-    string8_v,
-    string16_v,
-    string32_v
-  };
-
   // -- constructors, destructors, and assignment operators --------------------
 
   data_processor(const data_processor&) = delete;
@@ -137,7 +119,7 @@ public:
   apply(T& x) {
     static constexpr auto tlindex = detail::tl_index_of<builtin_t, T>::value;
     static_assert(tlindex >= 0, "T not recognized as builtin type");
-    return apply_builtin(static_cast<builtin>(tlindex), &x);
+    return apply_impl(x);
   }
 
   template <class T>
@@ -147,25 +129,21 @@ public:
     error
   >::type
   apply(T& x) {
-    using type =
-      typename detail::select_integer_type<
-        static_cast<int>(sizeof(T)) * (std::is_signed<T>::value ? -1 : 1)
-      >::type;
-    static constexpr auto tlindex = detail::tl_index_of<builtin_t, type>::value;
-    static_assert(tlindex >= 0, "T not recognized as builtin type");
-    return apply_builtin(static_cast<builtin>(tlindex), &x);
+    using type = detail::select_integer_type_t<sizeof(T),
+                                               std::is_signed<T>::value>;
+    return apply_impl(reinterpret_cast<type&>(x));
   }
 
   error apply(std::string& x) {
-    return apply_builtin(string8_v, &x);
+    return apply_impl(x);
   }
 
   error apply(std::u16string& x) {
-    return apply_builtin(string16_v, &x);
+    return apply_impl(x);
   }
 
   error apply(std::u32string& x) {
-    return apply_builtin(string32_v, &x);
+    return apply_impl(x);
   }
 
   template <class D, atom_value V>
@@ -198,7 +176,7 @@ public:
         lhs = static_cast<underlying>(rhs);
       }
     } assign;
-    underlying tmp;
+    underlying tmp = 0;
     return convert_apply(dref(), x, tmp, assign);
   }
 
@@ -249,7 +227,7 @@ public:
         lhs.resize((rhs.size() - 1) / 8 + 1, 0);
         for (bool b: rhs) {
           if (b)
-            lhs[k / 8] |= (1 << (k % 8));
+            lhs[k / 8] |= static_cast<uint8_t>(1 << (k % 8));
           ++k;
         }
       }
@@ -262,12 +240,10 @@ public:
   template <class T>
   error consume_range(T& xs) {
     for (auto& x : xs) {
-      using value_type = typename std::remove_const<
-                           typename std::remove_reference<decltype(x)>::type
-                         >::type;
-      auto e = apply(const_cast<value_type&>(x));
-      if (e)
-        return e;
+      using value_type = typename std::remove_reference<decltype(x)>::type;
+      using mutable_type = typename std::remove_const<value_type>::type;
+      if (auto err = dref().apply(const_cast<mutable_type&>(x)))
+        return err;
     }
     return none;
   }
@@ -276,9 +252,8 @@ public:
   template <class U, class T>
   error consume_range_c(T& xs) {
     for (U x : xs) {
-      auto e = apply(x);
-      if (e)
-        return e;
+      if (auto err = dref().apply(x))
+        return err;
     }
     return none;
   }
@@ -289,8 +264,7 @@ public:
     auto insert_iter = std::inserter(xs, xs.end());
     for (size_t i = 0; i < num_elements; ++i) {
       typename std::remove_const<typename T::value_type>::type x;
-      auto err = apply(x);
-      if (err)
+      if (auto err = dref().apply(x))
         return err;
       *insert_iter++ = std::move(x);
     }
@@ -304,8 +278,7 @@ public:
     auto insert_iter = std::inserter(xs, xs.end());
     for (size_t i = 0; i < num_elements; ++i) {
       U x;
-      auto err = apply(x);
-      if (err)
+      if (auto err = dref().apply(x))
         return err;
       *insert_iter++ = std::move(x);
     }
@@ -402,8 +375,9 @@ public:
     using t0 = typename std::remove_const<F>::type;
     // This cast allows the data processor to cope with
     // `pair<const K, V>` value types used by `std::map`.
-    return error::eval([&] { return apply(const_cast<t0&>(xs.first)); },
-                       [&] { return apply(xs.second); });
+    if (auto err = dref().apply(const_cast<t0&>(xs.first)))
+      return err;
+    return dref().apply(xs.second);
   }
 
   template <class... Ts>
@@ -468,8 +442,8 @@ public:
     return convert_apply(dref(), x, tmp, assign);
   }
 
-  template <class Duration>
-  error apply(std::chrono::time_point<std::chrono::system_clock, Duration>& t) {
+  template <class Clock, class Duration>
+  error apply(std::chrono::time_point<Clock, Duration>& t) {
     if (Derived::reads_state) {
       auto dur = t.time_since_epoch();
       return apply(dur);
@@ -477,7 +451,7 @@ public:
     if (Derived::writes_state) {
       Duration dur{};
       auto e = apply(dur);
-      t = std::chrono::time_point<std::chrono::system_clock, Duration>{dur};
+      t = std::chrono::time_point<Clock, Duration>{dur};
       return e;
     }
   }
@@ -497,29 +471,68 @@ public:
 
   // -- operator() -------------------------------------------------------------
 
-  inline error operator()() {
+  error operator()() {
     return none;
   }
 
+#if defined(__cpp_fold_expressions) && defined(__cpp_if_constexpr)
+
+  template <class... Ts>
+  error operator()(Ts&&... xs) {
+    error result;
+    auto f = [&result, this](auto&& x) {
+      using type = detail::decay_t<decltype(x)>;
+      if constexpr (meta::is_save_callback<type>::value) {
+        if constexpr (Derived::reads_state)
+          if (auto err = x.fun()) {
+            result = std::move(err);
+            return false;
+          }
+      } else if constexpr (meta::is_load_callback<type>::value) {
+        if constexpr (Derived::writes_state)
+          if (auto err = x.fun()) {
+            result = std::move(err);
+            return false;
+          }
+      } else if constexpr (meta::is_annotation<type>::value
+                           || is_allowed_unsafe_message_type<type>::value) {
+        // skip element
+      } else {
+        if (auto err = dref().apply(deconst(x))) {
+          result = std::move(err);
+          return false;
+        }
+      }
+      return true;
+    };
+    if ((f(std::forward<Ts>(xs)) && ...))
+      return none;
+    return result;
+  }
+
+#else // defined(__cpp_fold_expressions) && defined(__cpp_if_constexpr)
+
   template <class F, class... Ts>
   error operator()(meta::save_callback_t<F> x, Ts&&... xs) {
-    error e;
+    // TODO: use `if constexpr` when switching to C++17.
     if (Derived::reads_state)
-      e = x.fun();
-    return e ? e : (*this)(std::forward<Ts>(xs)...);
+      if (auto err = x.fun())
+        return err;
+    return dref()(std::forward<Ts>(xs)...);
   }
 
   template <class F, class... Ts>
   error operator()(meta::load_callback_t<F> x, Ts&&... xs) {
-    error e;
+    // TODO: use `if constexpr` when switching to C++17.
     if (Derived::writes_state)
-      e = x.fun();
-    return e ? e : (*this)(std::forward<Ts>(xs)...);
+      if (auto err = x.fun())
+        return err;
+    return dref()(std::forward<Ts>(xs)...);
   }
 
   template <class... Ts>
   error operator()(const meta::annotation&, Ts&&... xs) {
-    return (*this)(std::forward<Ts>(xs)...);
+    return dref()(std::forward<Ts>(xs)...);
   }
 
   template <class T, class... Ts>
@@ -528,7 +541,7 @@ public:
     error
   >::type
   operator()(const T&, Ts&&... xs) {
-    return (*this)(std::forward<Ts>(xs)...);
+    return dref()(std::forward<Ts>(xs)...);
   }
 
   template <class T, class... Ts>
@@ -544,13 +557,41 @@ public:
                              typename std::remove_reference<T>::type
                            >::value),
                   "a loading inspector requires mutable lvalue references");
-    auto e = apply(deconst(x));
-    return e ? e : (*this)(std::forward<Ts>(xs)...);
+    if (auto err = apply(deconst(x)))
+      return err;
+    return dref()(std::forward<Ts>(xs)...);
   }
 
+#endif // defined(__cpp_fold_expressions) && defined(__cpp_if_constexpr)
+
 protected:
-  /// Applies this processor to a single builtin value.
-  virtual error apply_builtin(builtin in_out_type, void* in_out) = 0;
+  virtual error apply_impl(int8_t&) = 0;
+
+  virtual error apply_impl(uint8_t&) = 0;
+
+  virtual error apply_impl(int16_t&) = 0;
+
+  virtual error apply_impl(uint16_t&) = 0;
+
+  virtual error apply_impl(int32_t&) = 0;
+
+  virtual error apply_impl(uint32_t&) = 0;
+
+  virtual error apply_impl(int64_t&) = 0;
+
+  virtual error apply_impl(uint64_t&) = 0;
+
+  virtual error apply_impl(float&) = 0;
+
+  virtual error apply_impl(double&) = 0;
+
+  virtual error apply_impl(long double&) = 0;
+
+  virtual error apply_impl(std::string&) = 0;
+
+  virtual error apply_impl(std::u16string&) = 0;
+
+  virtual error apply_impl(std::u32string&) = 0;
 
 private:
   template <class T>
@@ -562,15 +603,14 @@ private:
   static typename std::enable_if<D::reads_state, error>::type
   convert_apply(D& self, T& x, U& storage, F assign) {
     assign(storage, x);
-    return self.apply(storage);
+    return self(storage);
   }
 
   template <class D, class T, class U, class F>
   static typename std::enable_if<!D::reads_state, error>::type
   convert_apply(D& self, T& x, U& storage, F assign) {
-    auto e = self.apply(storage);
-    if (e)
-      return e;
+    if (auto err = self(storage))
+      return err;
     assign(x, storage);
     return none;
   }
@@ -584,4 +624,3 @@ private:
 };
 
 } // namespace caf
-
