@@ -33,30 +33,59 @@
 #include "caf/detail/type_list.hpp"
 
 #define CAF_HAS_MEMBER_TRAIT(name)                                             \
-template <class T>                                                             \
-struct has_##name##_member {                                                   \
-  template <class U>                                                           \
-  static auto sfinae(U* x) -> decltype(x->name(), std::true_type());           \
+  template <class T>                                                           \
+  class has_##name##_member {                                                  \
+  private:                                                                     \
+    template <class U>                                                         \
+    static auto sfinae(U* x) -> decltype(x->name(), std::true_type());         \
                                                                                \
-  template <class U>                                                           \
-  static auto sfinae(...) -> std::false_type;                                  \
+    template <class U>                                                         \
+    static auto sfinae(...) -> std::false_type;                                \
                                                                                \
-  using type = decltype(sfinae<T>(nullptr));                                   \
-  static constexpr bool value = type::value;                                   \
-}
+    using sfinae_type = decltype(sfinae<T>(nullptr));                          \
+                                                                               \
+  public:                                                                      \
+    static constexpr bool value = sfinae_type::value;                          \
+  }
+
+#define CAF_HAS_ALIAS_TRAIT(name)                                              \
+  template <class T>                                                           \
+  class has_##name##_alias {                                                   \
+  private:                                                                     \
+    template <class C>                                                         \
+    static std::true_type sfinae(typename C::name*);                           \
+                                                                               \
+    template <class>                                                           \
+    static std::false_type sfinae(...);                                        \
+                                                                               \
+    using sfinae_type = decltype(sfinae<T>(nullptr));                          \
+                                                                               \
+  public:                                                                      \
+    static constexpr bool value = sfinae_type::value;                          \
+  }
 
 namespace caf {
 namespace detail {
 
+// -- backport of C++14 additions ----------------------------------------------
+
 template <class T>
 using decay_t = typename std::decay<T>::type;
 
+template <bool B, class T, class F>
+using conditional_t = typename std::conditional<B, T, F>::type;
+
 template <bool V, class T = void>
 using enable_if_t = typename std::enable_if<V, T>::type;
 
+// -- custom traits ------------------------------------------------------------
+
 template <class Trait, class T = void>
 using enable_if_tt = typename std::enable_if<Trait::value, T>::type;
 
+template <class T>
+using remove_reference_t = typename std::remove_reference<T>::type;
+
 /// Checks whether `T` is inspectable by `Inspector`.
 template <class Inspector, class T>
 class is_inspectable {
@@ -158,6 +187,13 @@ struct is_primitive {
                                 || std::is_convertible<T, atom_value>::value;
 };
 
+// Workaround for weird GCC 4.8 STL implementation that breaks
+// `std::is_convertible<T, atom_value>::value` for tuples containing atom
+// constants.
+// TODO: remove when dropping support for GCC 4.8.
+template <class... Ts>
+struct is_primitive<std::tuple<Ts...>> : std::false_type {};
+
 /// Checks whether `T1` is comparable with `T2`.
 template <class T1, class T2>
 class is_comparable {
@@ -323,6 +359,17 @@ struct is_serializable_impl<T, false, false> {
                                 || is_builtin<T>::value;
 };
 
+template <class Rep, class Period>
+struct is_serializable_impl<std::chrono::duration<Rep, Period>, false, false> {
+  static constexpr bool value = is_serializable<Rep>::value;
+};
+
+template <class Clock, class Duration>
+struct is_serializable_impl<std::chrono::time_point<Clock, Duration>, false,
+                            false> {
+  static constexpr bool value = is_serializable<Duration>::value;
+};
+
 template <class F, class S>
 struct is_serializable_impl<std::pair<F, S>, false, false> {
   static constexpr bool value = is_serializable<F>::value
@@ -409,6 +456,7 @@ struct callable_trait<R (Ts...)> {
   using arg_types = type_list<Ts...>;
   using fun_sig = R (Ts...);
   using fun_type = std::function<R (Ts...)>;
+  static constexpr size_t num_args = sizeof...(Ts);
 };
 
 // member const function pointer
@@ -423,6 +471,23 @@ struct callable_trait<R (C::*)(Ts...)> : callable_trait<R (Ts...)> {};
 template <class R, class... Ts>
 struct callable_trait<R (*)(Ts...)> : callable_trait<R (Ts...)> {};
 
+#if __cplusplus >= 201703L
+
+// member const function pointer
+template <class C, typename R, class... Ts>
+struct callable_trait<R (C::*)(Ts...) const noexcept>
+  : callable_trait<R(Ts...)> {};
+
+// member function pointer
+template <class C, typename R, class... Ts>
+struct callable_trait<R (C::*)(Ts...) noexcept> : callable_trait<R(Ts...)> {};
+
+// good ol' function pointer
+template <class R, class... Ts>
+struct callable_trait<R (*)(Ts...) noexcept> : callable_trait<R(Ts...)> {};
+
+#endif
+
 template <class T>
 struct has_apply_operator {
   template <class U>
@@ -472,6 +537,9 @@ struct get_callable_trait_helper<T, false, false> {};
 template <class T>
 struct get_callable_trait : get_callable_trait_helper<decay_t<T>> {};
 
+template <class T>
+using get_callable_trait_t = typename get_callable_trait<T>::type;
+
 /// Checks wheter `T` is a function or member function.
 template <class T>
 struct is_callable {
@@ -570,6 +638,7 @@ public:
 
 CAF_HAS_MEMBER_TRAIT(size);
 CAF_HAS_MEMBER_TRAIT(data);
+CAF_HAS_MEMBER_TRAIT(clear);
 
 /// Checks whether F is convertible to either `std::function<void (T&)>`
 /// or `std::function<void (const T&)>`.
@@ -669,10 +738,13 @@ template <class T>
 struct is_expected<expected<T>> : std::true_type {};
 
 // Checks whether `T` and `U` are integers of the same size and signedness.
+// clang-format off
 template <class T, class U,
           bool Enable = std::is_integral<T>::value
                         && std::is_integral<U>::value
-                        && !std::is_same<T, bool>::value>
+                        && !std::is_same<T, bool>::value
+                        && !std::is_same<U, bool>::value>
+// clang-format on
 struct is_equal_int_type {
   static constexpr bool value = sizeof(T) == sizeof(U)
                                 && std::is_signed<T>::value
@@ -698,6 +770,101 @@ struct is_same_ish
 template <class>
 struct always_false : std::false_type {};
 
+/// Utility trait for removing const inside a `map<K, V>::value_type`.
+template <class T>
+struct deconst_kvp {
+  using type = T;
+};
+
+template <class K, class V>
+struct deconst_kvp<std::pair<const K, V>> {
+  using type = std::pair<K, V>;
+};
+
+template <class T>
+using deconst_kvp_t = typename deconst_kvp<T>::type;
+
+/// Utility trait for checking whether T is a `std::pair`.
+template <class T>
+struct is_pair : std::false_type {};
+
+/// Utility trait for checking whether T is a `std::pair`.
+template <class First, class Second>
+struct is_pair<std::pair<First, Second>> : std::true_type {};
+
+// -- traits to check for STL-style type aliases -------------------------------
+
+CAF_HAS_ALIAS_TRAIT(value_type);
+
+CAF_HAS_ALIAS_TRAIT(key_type);
+
+CAF_HAS_ALIAS_TRAIT(mapped_type);
+
+// -- constexpr functions for use in enable_if & friends -----------------------
+
+template <class List1, class List2>
+struct all_constructible : std::false_type {};
+
+template <>
+struct all_constructible<type_list<>, type_list<>> : std::true_type {};
+
+template <class T, class... Ts, class U, class... Us>
+struct all_constructible<type_list<T, Ts...>, type_list<U, Us...>> {
+  static constexpr bool value = std::is_constructible<T, U>::value
+                                && all_constructible<type_list<Ts...>,
+                                                     type_list<Us...>>::value;
+};
+
+/// Checks whether T behaves like `std::map`.
+template <class T>
+struct is_map_like {
+  static constexpr bool value = is_iterable<T>::value
+                                && has_key_type_alias<T>::value
+                                && has_mapped_type_alias<T>::value;
+};
+
+/// Checks whether T behaves like `std::vector`, `std::list`, or `std::set`.
+template <class T>
+struct is_list_like {
+  static constexpr bool value = is_iterable<T>::value
+                                && has_value_type_alias<T>::value
+                                && !has_mapped_type_alias<T>::value;
+};
+
+template <class F, class... Ts>
+struct is_invocable {
+private:
+  template <class U>
+  static auto sfinae(U* f)
+    -> decltype((*f)(std::declval<Ts>()...), std::true_type());
+
+  template <class U>
+  static auto sfinae(...) -> std::false_type;
+
+  using sfinae_type = decltype(sfinae<F>(nullptr));
+
+public:
+  static constexpr bool value = sfinae_type::value;
+};
+
+template <class R, class F, class... Ts>
+struct is_invocable_r {
+private:
+  template <class U>
+  static auto sfinae(U* f)
+    -> std::is_same<R, decltype((*f)(std::declval<Ts>()...))>;
+
+  template <class U>
+  static auto sfinae(...) -> std::false_type;
+
+  using sfinae_type = decltype(sfinae<F>(nullptr));
+
+public:
+  static constexpr bool value = sfinae_type::value;
+};
+
 } // namespace detail
 } // namespace caf
 
+#undef CAF_HAS_MEMBER_TRAIT
+#undef CAF_HAS_ALIAS_TRAIT
