@@ -30,8 +30,11 @@ class execution_unit {
 public:
   explicit execution_unit(actor_system* sys);
 
-  execution_unit(execution_unit&&) = delete;
-  execution_unit(const execution_unit&) = delete;
+  execution_unit() = default;
+  execution_unit(execution_unit&&) = default;
+  execution_unit& operator=(execution_unit&&) = default;
+  execution_unit(const execution_unit&) = default;
+  execution_unit& operator=(const execution_unit&) = default;
 
   virtual ~execution_unit();
 
@@ -58,8 +61,8 @@ public:
   }
 
 protected:
-  actor_system* system_;
-  proxy_registry* proxies_;
+  actor_system* system_ = nullptr;
+  proxy_registry* proxies_ = nullptr;
 };
 
 } // namespace caf
