@@ -22,6 +22,7 @@
 #include <map>
 #include <memory>
 #include <tuple>
+#include <vector>
 
 #include "caf/detail/is_one_of.hpp"
 #include "caf/detail/is_primitive_config_value.hpp"
@@ -29,6 +30,8 @@
 
 namespace caf {
 
+// clang-format off
+
 // -- 1 param templates --------------------------------------------------------
 
 template <class> class behavior_type_of;
@@ -39,6 +42,8 @@ template <class> class intrusive_cow_ptr;
 template <class> class intrusive_ptr;
 template <class> class optional;
 template <class> class param;
+template <class> class serializer_impl;
+template <class> class span;
 template <class> class stream;
 template <class> class stream_sink;
 template <class> class stream_source;
@@ -51,6 +56,8 @@ template <class> struct timeout_definition;
 
 template <class, class> class stream_stage;
 
+template <class Iterator, class Sentinel = Iterator> struct parser_state;
+
 // -- 3 param templates --------------------------------------------------------
 
 template <class, class, int> class actor_cast_access;
@@ -59,18 +66,16 @@ template <class, class, class> class broadcast_downstream_manager;
 
 // -- variadic templates -------------------------------------------------------
 
-template <class...> class result;
-template <class...> class variant;
+template <class...> class cow_tuple;
 template <class...> class delegated;
 template <class...> class result;
 template <class...> class typed_actor;
 template <class...> class typed_actor_pointer;
 template <class...> class typed_event_based_actor;
 template <class...> class typed_response_promise;
+template <class...> class variant;
 
-// -- variadic templates with fixed arguments ----------------------------------
-//
-template <class, class...> class output_stream;
+// clang-format on
 
 // -- classes ------------------------------------------------------------------
 
@@ -83,11 +88,13 @@ class actor_companion;
 class actor_config;
 class actor_control_block;
 class actor_pool;
+class actor_profiler;
 class actor_proxy;
 class actor_registry;
 class actor_system;
 class actor_system_config;
 class behavior;
+class binary_deserializer;
 class blocking_actor;
 class config_option;
 class config_option_adder;
@@ -105,8 +112,10 @@ class group;
 class group_module;
 class inbound_path;
 class ipv4_address;
+class ipv4_endpoint;
 class ipv4_subnet;
 class ipv6_address;
+class ipv6_endpoint;
 class ipv6_subnet;
 class local_actor;
 class mailbox_element;
@@ -134,7 +143,8 @@ class uri_builder;
 
 // -- templates with default parameters ----------------------------------------
 
-template <class, class = event_based_actor> class stateful_actor;
+template <class, class = event_based_actor>
+class stateful_actor;
 
 // -- structs ------------------------------------------------------------------
 
@@ -164,22 +174,28 @@ config_option make_config_option(T& storage, string_view category,
 
 // -- enums --------------------------------------------------------------------
 
-enum class stream_priority;
 enum class atom_value : uint64_t;
+enum class byte : uint8_t;
+enum class exit_reason : uint8_t;
+enum class invoke_message_result;
+enum class pec : uint8_t;
+enum class sec : uint8_t;
+enum class stream_priority;
 
 // -- aliases ------------------------------------------------------------------
 
 using actor_id = uint64_t;
+using binary_serializer = serializer_impl<std::vector<char>>;
 using ip_address = ipv6_address;
+using ip_endpoint = ipv6_endpoint;
 using ip_subnet = ipv6_subnet;
+using settings = dictionary<config_value>;
 using stream_slot = uint16_t;
 
-using config_value_map = dictionary<dictionary<config_value>>;
-
 // -- functions ----------------------------------------------------------------
 
 /// @relates actor_system_config
-const dictionary<dictionary<config_value>>& content(const actor_system_config&);
+const settings& content(const actor_system_config&);
 
 // -- intrusive containers -----------------------------------------------------
 
@@ -213,6 +229,14 @@ struct header;
 
 } // namespace io
 
+// -- networking classes -------------------------------------------------------
+
+namespace net {
+
+class middleman;
+
+} // namespace net
+
 // -- OpenCL classes -----------------------------------------------------------
 
 namespace opencl {
@@ -231,7 +255,6 @@ class abstract_coordinator;
 
 } // namespace scheduler
 
-
 // -- OpenSSL classes ----------------------------------------------------------
 
 namespace openssl {
@@ -244,9 +267,13 @@ class manager;
 
 namespace detail {
 
-template <class> class type_erased_value_impl;
-template <class> class stream_distribution_tree;
+template <class>
+class type_erased_value_impl;
+template <class>
+class stream_distribution_tree;
 
+class abstract_worker;
+class abstract_worker_hub;
 class disposer;
 class dynamic_message_data;
 class group_manager;
@@ -280,4 +307,3 @@ using type_erased_value_ptr = std::unique_ptr<type_erased_value>;
 using mailbox_element_ptr = std::unique_ptr<mailbox_element, detail::disposer>;
 
 } // namespace caf
-
