@@ -21,11 +21,12 @@
 #include <vector>
 
 #include "caf/actor.hpp"
-#include "caf/message.hpp"
 #include "caf/actor_addr.hpp"
+#include "caf/actor_cast.hpp"
+#include "caf/check_typed_input.hpp"
+#include "caf/message.hpp"
 #include "caf/message_id.hpp"
 #include "caf/response_type.hpp"
-#include "caf/check_typed_input.hpp"
 
 namespace caf {
 
@@ -56,15 +57,15 @@ public:
   detail::enable_if_t<((sizeof...(Ts) > 0)
                        || (!std::is_convertible<T, error>::value
                            && !std::is_same<detail::decay_t<T>, unit_t>::value))
-                        && !detail::is_expected<detail::decay_t<T>>::value>
+                      && !detail::is_expected<detail::decay_t<T>>::value>
   deliver(T&& x, Ts&&... xs) {
     using ts = detail::type_list<detail::decay_t<T>, detail::decay_t<Ts>...>;
     static_assert(!detail::tl_exists<ts, detail::is_result>::value,
                   "it is not possible to deliver objects of type result<T>");
     static_assert(!detail::tl_exists<ts, detail::is_expected>::value,
                   "mixing expected<T> with regular values is not supported");
-    return deliver_impl(make_message(std::forward<T>(x),
-                                     std::forward<Ts>(xs)...));
+    return deliver_impl(
+      make_message(std::forward<T>(x), std::forward<Ts>(xs)...));
   }
 
   template <class T>
@@ -75,28 +76,22 @@ public:
   }
 
   /// Satisfies the promise by delegating to another actor.
-  template <message_priority P = message_priority::normal,
-           class Handle = actor, class... Ts>
-   typename response_type<
-    typename Handle::signatures,
-    detail::implicit_conversions_t<typename std::decay<Ts>::type>...
-  >::delegated_type
+  template <message_priority P = message_priority::normal, class Handle = actor,
+            class... Ts>
+  typename response_type<typename Handle::signatures,
+                         detail::implicit_conversions_t<
+                           typename std::decay<Ts>::type>...>::delegated_type
   delegate(const Handle& dest, Ts&&... xs) {
     static_assert(sizeof...(Ts) > 0, "nothing to delegate");
-    using token =
-      detail::type_list<
-        typename detail::implicit_conversions<
-          typename std::decay<Ts>::type
-        >::type...>;
+    using token = detail::type_list<typename detail::implicit_conversions<
+      typename std::decay<Ts>::type>::type...>;
     static_assert(response_type_unbox<signatures_of_t<Handle>, token>::valid,
                   "receiver does not accept given message");
-    if (dest) {
-      auto mid = P == message_priority::high ? id_.with_high_priority() : id_;
-      dest->enqueue(make_mailbox_element(std::move(source_), mid,
-                                         std::move(stages_),
-                                         std::forward<Ts>(xs)...),
-                    context());
-    }
+    // TODO: use `if constexpr` when switching to C++17
+    if (P == message_priority::high)
+      id_ = id_.with_high_priority();
+    delegate_impl(actor_cast<abstract_actor*>(dest),
+                  make_message(std::forward<Ts>(xs)...));
     return {};
   }
 
@@ -111,36 +106,41 @@ public:
   bool async() const;
 
   /// Queries whether this promise is a valid promise that is not satisfied yet.
-  inline bool pending() const {
+  bool pending() const {
     return source_ != nullptr || !stages_.empty();
   }
 
   /// Returns the source of the corresponding request.
-  inline const strong_actor_ptr& source() const {
+  const strong_actor_ptr& source() const {
     return source_;
   }
 
   /// Returns the remaining stages for the corresponding request.
-  inline const forwarding_stack& stages() const {
+  const forwarding_stack& stages() const {
     return stages_;
   }
 
   /// Returns the actor that will receive the response, i.e.,
   /// `stages().front()` if `!stages().empty()` or `source()` otherwise.
-  inline strong_actor_ptr next() const {
+  strong_actor_ptr next() const {
     return stages_.empty() ? source_ : stages_.front();
   }
 
   /// Returns the message ID of the corresponding request.
-  inline message_id id() const {
+  message_id id() const {
     return id_;
   }
 
 private:
+  /// Returns a downcasted version of `self_`.
+  local_actor* self_dptr() const;
+
   execution_unit* context();
 
   void deliver_impl(message msg);
 
+  void delegate_impl(abstract_actor* receiver, message msg);
+
   strong_actor_ptr self_;
   strong_actor_ptr source_;
   forwarding_stack stages_;
