@@ -31,7 +31,6 @@
 #include "caf/stateful_actor.hpp"
 #include "caf/typed_behavior.hpp"
 #include "caf/typed_response_promise.hpp"
-#include "caf/unsafe_actor_handle_init.hpp"
 
 #include "caf/detail/mpi_splice.hpp"
 #include "caf/decorator/splitter.hpp"
@@ -142,16 +141,11 @@ class typed_actor : detail::comparable<typed_actor<Sigs...>>,
   }
 
   // allow `handle_type{this}` for typed actors
-  template <class T>
-  typed_actor(T* ptr,
-              typename std::enable_if<
-                std::is_base_of<statically_typed_actor_base, T>::value
-              >::type* = 0)
-      : ptr_(ptr->ctrl()) {
-    static_assert(detail::tl_subset_of<
-                    signatures,
-                    typename T::signatures
-                  >::value,
+  template <class T,
+            class = detail::enable_if_t<actor_traits<T>::is_statically_typed>>
+  typed_actor(T* ptr) : ptr_(ptr->ctrl()) {
+    static_assert(detail::tl_subset_of<signatures,
+                                       typename T::signatures>::value,
                   "Cannot assign T* to incompatible handle type");
     CAF_ASSERT(ptr != nullptr);
   }
@@ -172,10 +166,6 @@ class typed_actor : detail::comparable<typed_actor<Sigs...>>,
     return *this;
   }
 
-  explicit typed_actor(const unsafe_actor_handle_init_t&) CAF_DEPRECATED {
-    // nop
-  }
-
   /// Queries whether this actor handle is valid.
   inline explicit operator bool() const {
     return static_cast<bool>(ptr_);
@@ -211,12 +201,6 @@ class typed_actor : detail::comparable<typed_actor<Sigs...>>,
     ptr_.swap(other.ptr_);
   }
 
-  /// Queries whether this object was constructed using
-  /// `unsafe_actor_handle_init` or is in moved-from state.
-  bool unsafe() const CAF_DEPRECATED {
-    return !ptr_;
-  }
-
   /// @cond PRIVATE
 
   abstract_actor* operator->() const noexcept {
