@@ -39,7 +39,6 @@
 #include "caf/actor_control_block.hpp"
 
 #include "caf/detail/disposer.hpp"
-#include "caf/detail/enum_to_string.hpp"
 #include "caf/detail/shared_spinlock.hpp"
 
 namespace caf {
@@ -90,6 +89,10 @@ actor_system& abstract_actor::home_system() const noexcept {
   return *(actor_control_block::from(this)->home_system);
 }
 
+mailbox_element* abstract_actor::peek_at_next_mailbox_element() {
+  return nullptr;
+}
+
 void abstract_actor::register_at_system() {
   if (getf(is_registered_flag))
     return;
