@@ -18,38 +18,35 @@
 
 #include "caf/actor_ostream.hpp"
 
-#include "caf/send.hpp"
-#include "caf/scoped_actor.hpp"
 #include "caf/abstract_actor.hpp"
 #include "caf/default_attachable.hpp"
+#include "caf/scoped_actor.hpp"
+#include "caf/send.hpp"
 
 #include "caf/scheduler/abstract_coordinator.hpp"
 
 namespace caf {
 
 actor_ostream::actor_ostream(local_actor* self)
-    : self_(self->id()),
-      printer_(self->home_system().scheduler().printer()) {
+  : self_(self->id()), printer_(self->home_system().scheduler().printer()) {
   init(self);
 }
 
 actor_ostream::actor_ostream(scoped_actor& self)
-    : self_(self->id()),
-      printer_(self->home_system().scheduler().printer()) {
+  : self_(self->id()), printer_(self->home_system().scheduler().printer()) {
   init(actor_cast<abstract_actor*>(self));
 }
 
 actor_ostream& actor_ostream::write(std::string arg) {
   printer_->enqueue(make_mailbox_element(nullptr, make_message_id(), {},
-                                         add_atom::value, self_,
-                                         std::move(arg)),
+                                         add_atom_v, self_, std::move(arg)),
                     nullptr);
   return *this;
 }
 
 actor_ostream& actor_ostream::flush() {
   printer_->enqueue(make_mailbox_element(nullptr, make_message_id(), {},
-                                          flush_atom::value, self_),
+                                         flush_atom_v, self_),
                     nullptr);
   return *this;
 }
@@ -59,16 +56,15 @@ void actor_ostream::redirect(abstract_actor* self, std::string fn, int flags) {
     return;
   auto pr = self->home_system().scheduler().printer();
   pr->enqueue(make_mailbox_element(nullptr, make_message_id(), {},
-                                    redirect_atom::value, self->id(),
-                                    std::move(fn), flags),
+                                   redirect_atom_v, self->id(), std::move(fn),
+                                   flags),
               nullptr);
 }
 
 void actor_ostream::redirect_all(actor_system& sys, std::string fn, int flags) {
   auto pr = sys.scheduler().printer();
   pr->enqueue(make_mailbox_element(nullptr, make_message_id(), {},
-                                    redirect_atom::value,
-                                    std::move(fn), flags),
+                                   redirect_atom_v, std::move(fn), flags),
               nullptr);
 }
 
