@@ -30,11 +30,11 @@ namespace caf {
 forwarding_actor_proxy::forwarding_actor_proxy(actor_config& cfg, actor dest)
     : actor_proxy(cfg),
       broker_(std::move(dest)) {
-  // nop
+  anon_send(broker_, monitor_atom_v, ctrl());
 }
 
 forwarding_actor_proxy::~forwarding_actor_proxy() {
-  anon_send(broker_, make_message(delete_atom::value, node(), id()));
+  anon_send(broker_, make_message(delete_atom_v, node(), id()));
 }
 
 void forwarding_actor_proxy::forward_msg(strong_actor_ptr sender,
@@ -48,7 +48,7 @@ void forwarding_actor_proxy::forward_msg(strong_actor_ptr sender,
   shared_lock<detail::shared_spinlock> guard(broker_mtx_);
   if (broker_)
     broker_->enqueue(nullptr, make_message_id(),
-                     make_message(forward_atom::value, std::move(sender),
+                     make_message(forward_atom_v, std::move(sender),
                                   fwd != nullptr ? *fwd : tmp,
                                   strong_actor_ptr{ctrl()}, mid,
                                   std::move(msg)),
@@ -66,7 +66,7 @@ void forwarding_actor_proxy::enqueue(mailbox_element_ptr what,
 bool forwarding_actor_proxy::add_backlink(abstract_actor* x) {
   if (monitorable_actor::add_backlink(x)) {
     forward_msg(ctrl(), make_message_id(),
-                make_message(link_atom::value, x->ctrl()));
+                make_message(link_atom_v, x->ctrl()));
     return true;
   }
   return false;
@@ -75,14 +75,13 @@ bool forwarding_actor_proxy::add_backlink(abstract_actor* x) {
 bool forwarding_actor_proxy::remove_backlink(abstract_actor* x) {
   if (monitorable_actor::remove_backlink(x)) {
     forward_msg(ctrl(), make_message_id(),
-                make_message(unlink_atom::value, x->ctrl()));
+                make_message(unlink_atom_v, x->ctrl()));
     return true;
   }
   return false;
 }
 
 void forwarding_actor_proxy::kill_proxy(execution_unit* ctx, error rsn) {
-  CAF_ASSERT(ctx != nullptr);
   actor tmp;
   { // lifetime scope of guard
     std::unique_lock<detail::shared_spinlock> guard(broker_mtx_);
