@@ -47,8 +47,7 @@ result<message> reflect_and_quit(scheduled_actor* ptr, message_view& x) {
 result<message> print_and_drop(scheduled_actor* ptr, message_view& x) {
   CAF_LOG_WARNING("unexpected message" << CAF_ARG(x.content()));
   aout(ptr) << "*** unexpected message [id: " << ptr->id()
-            << ", name: " << ptr->name() << "]: "
-            << x.content().stringify()
+            << ", name: " << ptr->name() << "]: " << x.content().stringify()
             << std::endl;
   return sec::unexpected_message;
 }
@@ -90,7 +89,7 @@ void scheduled_actor::default_exit_handler(scheduled_actor* ptr, exit_msg& x) {
     default_error_handler(ptr, x.reason);
 }
 
-# ifndef CAF_NO_EXCEPTIONS
+#ifndef CAF_NO_EXCEPTIONS
 error scheduled_actor::default_exception_handler(pointer ptr,
                                                  std::exception_ptr& x) {
   CAF_ASSERT(x != nullptr);
@@ -98,8 +97,8 @@ error scheduled_actor::default_exception_handler(pointer ptr,
     std::rethrow_exception(x);
   } catch (const std::exception& e) {
     aout(ptr) << "*** unhandled exception: [id: " << ptr->id()
-              << ", name: " << ptr->name() << ", exception typeid: "
-              << typeid(e).name() << "]: " << e.what()
+              << ", name: " << ptr->name()
+              << ", exception typeid: " << typeid(e).name() << "]: " << e.what()
               << std::endl;
   } catch (...) {
     aout(ptr) << "*** unhandled exception: [id: " << ptr->id()
@@ -108,23 +107,24 @@ error scheduled_actor::default_exception_handler(pointer ptr,
   }
   return sec::runtime_error;
 }
-# endif // CAF_NO_EXCEPTIONS
+#endif // CAF_NO_EXCEPTIONS
 
 // -- constructors and destructors ---------------------------------------------
 
 scheduled_actor::scheduled_actor(actor_config& cfg)
-    : super(cfg),
-      mailbox_(unit, unit, unit, unit, unit),
-      timeout_id_(0),
-      default_handler_(print_and_drop),
-      error_handler_(default_error_handler),
-      down_handler_(default_down_handler),
-      exit_handler_(default_exit_handler),
-      private_thread_(nullptr)
-# ifndef CAF_NO_EXCEPTIONS
-      , exception_handler_(default_exception_handler)
-# endif // CAF_NO_EXCEPTIONS
-      {
+  : super(cfg),
+    mailbox_(unit, unit, unit, unit, unit),
+    timeout_id_(0),
+    default_handler_(print_and_drop),
+    error_handler_(default_error_handler),
+    down_handler_(default_down_handler),
+    exit_handler_(default_exit_handler),
+    private_thread_(nullptr)
+#ifndef CAF_NO_EXCEPTIONS
+    ,
+    exception_handler_(default_exception_handler)
+#endif // CAF_NO_EXCEPTIONS
+{
   auto& sys_cfg = home_system().config();
   auto interval = sys_cfg.stream_tick_duration();
   CAF_ASSERT(interval.count() > 0);
@@ -138,12 +138,12 @@ scheduled_actor::scheduled_actor(actor_config& cfg)
   credit_round_ticks_ = div(sys_cfg.stream_credit_round_interval, interval);
   CAF_ASSERT(credit_round_ticks_ > 0);
   CAF_LOG_DEBUG(CAF_ARG(interval) << CAF_ARG(max_batch_delay_ticks_)
-                << CAF_ARG(credit_round_ticks_));
+                                  << CAF_ARG(credit_round_ticks_));
 }
 
 scheduled_actor::~scheduled_actor() {
   // signalize to the private thread object that it is
-  // unrachable and can be destroyed as well
+  // unreachable and can be destroyed as well
   if (private_thread_ != nullptr)
     private_thread_->notify_self_destroyed();
 }
@@ -187,6 +187,9 @@ void scheduled_actor::enqueue(mailbox_element_ptr ptr, execution_unit* eu) {
       break;
   }
 }
+mailbox_element* scheduled_actor::peek_at_next_mailbox_element() {
+  return mailbox().closed() || mailbox().blocked() ? nullptr : mailbox().peek();
+}
 
 // -- overridden functions of local_actor --------------------------------------
 
@@ -274,16 +277,19 @@ struct upstream_msg_visitor {
   }
 };
 
-} // namespace <anonymous>
+} // namespace
 
-intrusive::task_result scheduled_actor::mailbox_visitor::
-operator()(size_t, upstream_queue&, mailbox_element& x) {
+intrusive::task_result
+scheduled_actor::mailbox_visitor::operator()(size_t, upstream_queue&,
+                                             mailbox_element& x) {
   CAF_ASSERT(x.content().type_token() == make_type_token<upstream_msg>());
   self->current_mailbox_element(&x);
   CAF_LOG_RECEIVE_EVENT((&x));
+  CAF_BEFORE_PROCESSING(self, x);
   auto& um = x.content().get_mutable_as<upstream_msg>(0);
   upstream_msg_visitor f{self, um};
   visit(f, um.content);
+  CAF_AFTER_PROCESSING(self, invoke_message_result::consumed);
   return ++handled_msgs < max_throughput ? intrusive::task_result::resume
                                          : intrusive::task_result::stop_all;
 }
@@ -299,6 +305,7 @@ struct downstream_msg_visitor {
 
   template <class T>
   intrusive::task_result operator()(T& x) {
+    CAF_LOG_TRACE(CAF_ARG(x));
     auto& inptr = q_ref.policy().handler;
     if (inptr == nullptr)
       return intrusive::task_result::stop;
@@ -307,9 +314,9 @@ struct downstream_msg_visitor {
     inptr->handle(x);
     // The sender slot can be 0. This is the case for forced_close or
     // forced_drop messages from stream aborters.
-    CAF_ASSERT(inptr->slots == dm.slots
-               || (dm.slots.sender == 0
-                   && dm.slots.receiver == inptr->slots.receiver));
+    CAF_ASSERT(
+      inptr->slots == dm.slots
+      || (dm.slots.sender == 0 && dm.slots.receiver == inptr->slots.receiver));
     // TODO: replace with `if constexpr` when switching to C++17
     if (std::is_same<T, downstream_msg::close>::value
         || std::is_same<T, downstream_msg::forced_close>::value) {
@@ -317,12 +324,12 @@ struct downstream_msg_visitor {
       qs_ref.erase_later(dm.slots.receiver);
       selfptr->erase_stream_manager(dm.slots.receiver);
       if (mgr->done()) {
+        CAF_LOG_DEBUG("path is done receiving and closes its manager");
         selfptr->erase_stream_manager(mgr);
         mgr->stop();
       }
       return intrusive::task_result::stop;
-    }
-    else if (mgr->done()) {
+    } else if (mgr->done()) {
       CAF_LOG_DEBUG("path is done receiving and closes its manager");
       selfptr->erase_stream_manager(mgr);
       mgr->stop();
@@ -332,19 +339,20 @@ struct downstream_msg_visitor {
   }
 };
 
-} // namespace <anonymous>
+} // namespace
 
-intrusive::task_result scheduled_actor::mailbox_visitor::
-operator()(size_t, downstream_queue& qs, stream_slot,
-           policy::downstream_messages::nested_queue_type& q,
-           mailbox_element& x) {
+intrusive::task_result scheduled_actor::mailbox_visitor::operator()(
+  size_t, downstream_queue& qs, stream_slot,
+  policy::downstream_messages::nested_queue_type& q, mailbox_element& x) {
   CAF_LOG_TRACE(CAF_ARG(x) << CAF_ARG(handled_msgs));
   self->current_mailbox_element(&x);
   CAF_LOG_RECEIVE_EVENT((&x));
+  CAF_BEFORE_PROCESSING(self, x);
   CAF_ASSERT(x.content().type_token() == make_type_token<downstream_msg>());
   auto& dm = x.content().get_mutable_as<downstream_msg>(0);
   downstream_msg_visitor f{self, qs, q, dm};
   auto res = visit(f, dm.content);
+  CAF_AFTER_PROCESSING(self, invoke_message_result::consumed);
   return ++handled_msgs < max_throughput ? res
                                          : intrusive::task_result::stop_all;
 }
@@ -356,9 +364,8 @@ scheduled_actor::mailbox_visitor::operator()(mailbox_element& x) {
     case activation_result::terminated:
       return intrusive::task_result::stop;
     case activation_result::success:
-      return ++handled_msgs < max_throughput
-             ? intrusive::task_result::resume
-             : intrusive::task_result::stop_all;
+      return ++handled_msgs < max_throughput ? intrusive::task_result::resume
+                                             : intrusive::task_result::stop_all;
     case activation_result::skipped:
       return intrusive::task_result::skip;
     default:
@@ -366,8 +373,8 @@ scheduled_actor::mailbox_visitor::operator()(mailbox_element& x) {
   }
 }
 
-resumable::resume_result
-scheduled_actor::resume(execution_unit* ctx, size_t max_throughput) {
+resumable::resume_result scheduled_actor::resume(execution_unit* ctx,
+                                                 size_t max_throughput) {
   CAF_PUSH_AID(id());
   CAF_LOG_TRACE(CAF_ARG(max_throughput));
   if (!activate(ctx))
@@ -466,7 +473,7 @@ void scheduled_actor::quit(error x) {
 uint64_t scheduled_actor::set_receive_timeout(actor_clock::time_point x) {
   CAF_LOG_TRACE(x);
   setf(has_timeout_flag);
-  return set_timeout(receive_atom::value, x);
+  return set_timeout(receive_atom_v, x);
 }
 
 uint64_t scheduled_actor::set_receive_timeout() {
@@ -481,7 +488,7 @@ uint64_t scheduled_actor::set_receive_timeout() {
   if (d.is_zero()) {
     // immediately enqueue timeout message if duration == 0s
     auto id = ++timeout_id_;
-    auto type = receive_atom::value;
+    auto type = receive_atom_v;
     eq_impl(make_message_id(), nullptr, context(), timeout_msg{type, id});
     return id;
   }
@@ -512,15 +519,13 @@ uint64_t scheduled_actor::set_stream_timeout(actor_clock::time_point x) {
     mgrs.emplace_back(kvp.second);
   std::sort(mgrs.begin(), mgrs.end());
   auto e = std::unique(mgrs.begin(), mgrs.end());
-  auto idle = [=](const stream_manager_ptr& y) {
-    return y->idle();
-  };
+  auto idle = [=](const stream_manager_ptr& y) { return y->idle(); };
   if (std::all_of(mgrs.begin(), e, idle)) {
     CAF_LOG_DEBUG("suppress stream timeout");
     return 0;
   }
   // Delegate call.
-  return set_timeout(stream_atom::value, x);
+  return set_timeout(stream_atom_v, x);
 }
 
 // -- message processing -------------------------------------------------------
@@ -545,21 +550,20 @@ scheduled_actor::categorize(mailbox_element& x) {
   auto& content = x.content();
   switch (content.type_token()) {
     case make_type_token<atom_value, atom_value, std::string>():
-      if (content.get_as<atom_value>(0) == sys_atom::value
-          && content.get_as<atom_value>(1) == get_atom::value) {
+      if (content.get_as<atom_value>(0) == sys_atom_v
+          && content.get_as<atom_value>(1) == get_atom_v) {
+        auto rp = make_response_promise();
+        if (!rp.pending()) {
+          CAF_LOG_WARNING("received anonymous ('get', 'sys', $key) message");
+          return message_category::internal;
+        }
         auto& what = content.get_as<std::string>(2);
         if (what == "info") {
           CAF_LOG_DEBUG("reply to 'info' message");
-          x.sender->enqueue(
-            make_mailbox_element(ctrl(), x.mid.response_id(),
-                                  {}, ok_atom::value, std::move(what),
-                                  strong_actor_ptr{ctrl()}, name()),
-            context());
+          rp.deliver(ok_atom_v, std::move(what), strong_actor_ptr{ctrl()},
+                     name());
         } else {
-          x.sender->enqueue(
-            make_mailbox_element(ctrl(), x.mid.response_id(),
-                                  {}, sec::unsupported_sys_key),
-            context());
+          rp.deliver(make_error(sec::unsupported_sys_key));
         }
         return message_category::internal;
       }
@@ -568,7 +572,7 @@ scheduled_actor::categorize(mailbox_element& x) {
       CAF_ASSERT(x.mid.is_async());
       auto& tm = content.get_as<timeout_msg>(0);
       auto tid = tm.timeout_id;
-      if (tm.type == receive_atom::value) {
+      if (tm.type == receive_atom_v) {
         CAF_LOG_DEBUG("handle ordinary timeout message");
         if (is_active_receive_timeout(tid) && !bhvr_stack_.empty())
           bhvr_stack_.back().handle_timeout();
@@ -614,9 +618,9 @@ scheduled_actor::categorize(mailbox_element& x) {
       return message_category::internal;
     }
     case make_type_token<open_stream_msg>(): {
-      return handle_open_stream_msg(x) != im_skipped
-             ? message_category::internal
-             : message_category::skipped;
+      return handle_open_stream_msg(x) != invoke_message_result::skipped
+               ? message_category::internal
+               : message_category::skipped;
     }
     default:
       return message_category::ordinary;
@@ -627,99 +631,108 @@ invoke_message_result scheduled_actor::consume(mailbox_element& x) {
   CAF_LOG_TRACE(CAF_ARG(x));
   current_element_ = &x;
   CAF_LOG_RECEIVE_EVENT(current_element_);
-  // Helper function for dispatching a message to a response handler.
-  using ptr_t = scheduled_actor*;
-  using fun_t = bool (*)(ptr_t, behavior&, mailbox_element&);
-  auto ordinary_invoke = [](ptr_t, behavior& f, mailbox_element& in) -> bool {
-    return f(in.content()) != none;
-  };
-  auto select_invoke_fun = [&]() -> fun_t {
-    return ordinary_invoke;
-  };
-  // Short-circuit awaited responses.
-  if (!awaited_responses_.empty()) {
-    auto invoke = select_invoke_fun();
-    auto& pr = awaited_responses_.front();
-    // skip all messages until we receive the currently awaited response
-    if (x.mid != pr.first)
-      return im_skipped;
-    auto f = std::move(pr.second);
-    awaited_responses_.pop_front();
-    if (!invoke(this, f, x)) {
-      // try again with error if first attempt failed
-      auto msg = make_message(make_error(sec::unexpected_response,
-                                         x.move_content_to_message()));
-      f(msg);
+  CAF_BEFORE_PROCESSING(this, x);
+  // Wrap the actual body for the function.
+  auto body = [this, &x] {
+    // Helper function for dispatching a message to a response handler.
+    using ptr_t = scheduled_actor*;
+    using fun_t = bool (*)(ptr_t, behavior&, mailbox_element&);
+    auto ordinary_invoke = [](ptr_t, behavior& f, mailbox_element& in) -> bool {
+      return f(in.content()) != none;
+    };
+    auto select_invoke_fun = [&]() -> fun_t { return ordinary_invoke; };
+    // Short-circuit awaited responses.
+    if (!awaited_responses_.empty()) {
+      auto invoke = select_invoke_fun();
+      auto& pr = awaited_responses_.front();
+      // skip all messages until we receive the currently awaited response
+      if (x.mid != pr.first)
+        return invoke_message_result::skipped;
+      auto f = std::move(pr.second);
+      awaited_responses_.pop_front();
+      if (!invoke(this, f, x)) {
+        // try again with error if first attempt failed
+        auto msg = make_message(
+          make_error(sec::unexpected_response, x.move_content_to_message()));
+        f(msg);
+      }
+      return invoke_message_result::consumed;
     }
-    return im_success;
-  }
-  // Handle multiplexed responses.
-  if (x.mid.is_response()) {
-    auto invoke = select_invoke_fun();
-    auto mrh = multiplexed_responses_.find(x.mid);
-    // neither awaited nor multiplexed, probably an expired timeout
-    if (mrh == multiplexed_responses_.end())
-      return im_dropped;
-    auto bhvr = std::move(mrh->second);
-    multiplexed_responses_.erase(mrh);
-    if (!invoke(this, bhvr, x)) {
-      // try again with error if first attempt failed
-      auto msg = make_message(make_error(sec::unexpected_response,
-                                         x.move_content_to_message()));
-      bhvr(msg);
+    // Handle multiplexed responses.
+    if (x.mid.is_response()) {
+      auto invoke = select_invoke_fun();
+      auto mrh = multiplexed_responses_.find(x.mid);
+      // neither awaited nor multiplexed, probably an expired timeout
+      if (mrh == multiplexed_responses_.end())
+        return invoke_message_result::dropped;
+      auto bhvr = std::move(mrh->second);
+      multiplexed_responses_.erase(mrh);
+      if (!invoke(this, bhvr, x)) {
+        // try again with error if first attempt failed
+        auto msg = make_message(
+          make_error(sec::unexpected_response, x.move_content_to_message()));
+        bhvr(msg);
+      }
+      return invoke_message_result::consumed;
     }
-    return im_success;
-  }
-  // Dispatch on the content of x.
-  switch (categorize(x)) {
-    case message_category::skipped:
-      return im_skipped;
-    case message_category::internal:
-      CAF_LOG_DEBUG("handled system message");
-      return im_success;
-    case message_category::ordinary: {
-      detail::default_invoke_result_visitor<scheduled_actor> visitor{this};
-      bool skipped = false;
-      auto had_timeout = getf(has_timeout_flag);
-      if (had_timeout)
-        unsetf(has_timeout_flag);
-      // restore timeout at scope exit if message was skipped
-      auto timeout_guard = detail::make_scope_guard([&] {
-        if (skipped && had_timeout)
-          setf(has_timeout_flag);
-      });
-      auto call_default_handler = [&] {
-        auto sres = call_handler(default_handler_, this, x);
-        switch (sres.flag) {
+    // Dispatch on the content of x.
+    switch (categorize(x)) {
+      case message_category::skipped:
+        return invoke_message_result::skipped;
+      case message_category::internal:
+        CAF_LOG_DEBUG("handled system message");
+        return invoke_message_result::consumed;
+      case message_category::ordinary: {
+        detail::default_invoke_result_visitor<scheduled_actor> visitor{this};
+        bool skipped = false;
+        auto had_timeout = getf(has_timeout_flag);
+        if (had_timeout)
+          unsetf(has_timeout_flag);
+        // restore timeout at scope exit if message was skipped
+        auto timeout_guard = detail::make_scope_guard([&] {
+          if (skipped && had_timeout)
+            setf(has_timeout_flag);
+        });
+        auto call_default_handler = [&] {
+          auto sres = call_handler(default_handler_, this, x);
+          switch (sres.flag) {
+            default:
+              break;
+            case rt_error:
+            case rt_value:
+              visitor.visit(sres);
+              break;
+            case rt_skip:
+              skipped = true;
+          }
+        };
+        if (bhvr_stack_.empty()) {
+          call_default_handler();
+          return !skipped ? invoke_message_result::consumed
+                          : invoke_message_result::skipped;
+        }
+        auto& bhvr = bhvr_stack_.back();
+        switch (bhvr(visitor, x.content())) {
           default:
             break;
-          case rt_error:
-          case rt_value:
-            visitor.visit(sres);
-            break;
-          case rt_skip:
+          case match_case::skip:
             skipped = true;
+            break;
+          case match_case::no_match:
+            call_default_handler();
         }
-      };
-      if (bhvr_stack_.empty()) {
-        call_default_handler();
-        return !skipped ? im_success : im_skipped;
+        return !skipped ? invoke_message_result::consumed
+                        : invoke_message_result::skipped;
       }
-      auto& bhvr = bhvr_stack_.back();
-      switch (bhvr(visitor, x.content())) {
-        default:
-          break;
-        case match_case::skip:
-          skipped = true;
-          break;
-        case match_case::no_match:
-          call_default_handler();
-      }
-      return !skipped ? im_success : im_skipped;
     }
-  }
-  // Unreachable.
-  CAF_CRITICAL("invalid message type");
+    // Unreachable.
+    CAF_CRITICAL("invalid message type");
+  };
+  // Post-process the returned value from the function body.
+  auto result = body();
+  CAF_AFTER_PROCESSING(this, result);
+  CAF_LOG_SKIP_OR_FINALIZE_EVENT(result);
+  return result;
 }
 
 /// Tries to consume `x`.
@@ -727,7 +740,7 @@ void scheduled_actor::consume(mailbox_element_ptr x) {
   switch (consume(*x)) {
     default:
       break;
-    case im_skipped:
+    case invoke_message_result::skipped:
       push_to_cache(std::move(x));
   }
 }
@@ -741,9 +754,9 @@ bool scheduled_actor::activate(execution_unit* ctx) {
     CAF_LOG_ERROR("activate called on a terminated actor");
     return false;
   }
-# ifndef CAF_NO_EXCEPTIONS
+#ifndef CAF_NO_EXCEPTIONS
   try {
-# endif // CAF_NO_EXCEPTIONS
+#endif // CAF_NO_EXCEPTIONS
     if (!getf(is_initialized_flag)) {
       initialize();
       if (finalize()) {
@@ -752,21 +765,20 @@ bool scheduled_actor::activate(execution_unit* ctx) {
       }
       CAF_LOG_DEBUG("initialized actor:" << CAF_ARG(name()));
     }
-# ifndef CAF_NO_EXCEPTIONS
-  }
-  catch (...) {
+#ifndef CAF_NO_EXCEPTIONS
+  } catch (...) {
     CAF_LOG_ERROR("actor died during initialization");
     auto eptr = std::current_exception();
     quit(call_handler(exception_handler_, this, eptr));
     finalize();
     return false;
   }
-# endif // CAF_NO_EXCEPTIONS
+#endif // CAF_NO_EXCEPTIONS
   return true;
 }
 
 auto scheduled_actor::activate(execution_unit* ctx, mailbox_element& x)
--> activation_result {
+  -> activation_result {
   CAF_LOG_TRACE(CAF_ARG(x));
   if (!activate(ctx))
     return activation_result::terminated;
@@ -778,38 +790,42 @@ auto scheduled_actor::activate(execution_unit* ctx, mailbox_element& x)
 
 auto scheduled_actor::reactivate(mailbox_element& x) -> activation_result {
   CAF_LOG_TRACE(CAF_ARG(x));
-# ifndef CAF_NO_EXCEPTIONS
+#ifndef CAF_NO_EXCEPTIONS
+  auto handle_exception = [&](std::exception_ptr eptr) {
+    auto err = call_handler(exception_handler_, this, eptr);
+    if (x.mid.is_request()) {
+      auto rp = make_response_promise();
+      rp.deliver(err);
+    }
+    quit(std::move(err));
+  };
   try {
-# endif // CAF_NO_EXCEPTIONS
+#endif // CAF_NO_EXCEPTIONS
     switch (consume(x)) {
-      case im_dropped:
+      case invoke_message_result::dropped:
         return activation_result::dropped;
-      case im_success:
+      case invoke_message_result::consumed:
         bhvr_stack_.cleanup();
         if (finalize()) {
           CAF_LOG_DEBUG("actor finalized");
           return activation_result::terminated;
         }
         return activation_result::success;
-      case im_skipped:
+      case invoke_message_result::skipped:
         return activation_result::skipped;
     }
-# ifndef CAF_NO_EXCEPTIONS
-  }
-  catch (std::exception& e) {
+#ifndef CAF_NO_EXCEPTIONS
+  } catch (std::exception& e) {
     CAF_LOG_INFO("actor died because of an exception, what: " << e.what());
     static_cast<void>(e); // keep compiler happy when not logging
-    auto eptr = std::current_exception();
-    quit(call_handler(exception_handler_, this, eptr));
-  }
-  catch (...) {
+    handle_exception(std::current_exception());
+  } catch (...) {
     CAF_LOG_INFO("actor died because of an unknown exception");
-    auto eptr = std::current_exception();
-    quit(call_handler(exception_handler_, this, eptr));
+    handle_exception(std::current_exception());
   }
   finalize();
   return activation_result::terminated;
-# endif // CAF_NO_EXCEPTIONS
+#endif // CAF_NO_EXCEPTIONS
 }
 
 // -- behavior management ----------------------------------------------------
@@ -832,6 +848,21 @@ bool scheduled_actor::finalize() {
   // Repeated calls always return `true` but have no side effects.
   if (getf(is_cleaned_up_flag))
     return true;
+  // TODO: This is a workaround for issue #1011. Iterating over all stream
+  //       managers here and dropping them as needed prevents the
+  //       never-terminating part, but it still means that "dead" stream manager
+  //       can accumulate over time since we only run this O(n) path if the
+  //       actor is shutting down.
+  if (!has_behavior() && !stream_managers_.empty()) {
+    for (auto i = stream_managers_.begin(); i != stream_managers_.end();) {
+      if (i->second->done())
+        i = stream_managers_.erase(i);
+      else
+        ++i;
+      if (stream_managers_.empty())
+        stream_ticks_.stop();
+    }
+  }
   // An actor is considered alive as long as it has a behavior and didn't set
   // the terminated flag.
   if (alive())
@@ -879,14 +910,15 @@ scheduled_actor::urgent_queue& scheduled_actor::get_urgent_queue() {
 
 inbound_path* scheduled_actor::make_inbound_path(stream_manager_ptr mgr,
                                                  stream_slots slots,
-                                                 strong_actor_ptr sender) {
+                                                 strong_actor_ptr sender,
+                                                 rtti_pair rtti) {
   static constexpr size_t queue_index = downstream_queue_index;
   using policy_type = policy::downstream_messages::nested;
   auto& qs = get<queue_index>(mailbox_.queue().queues()).queues();
   auto res = qs.emplace(slots.receiver, policy_type{nullptr});
   if (!res.second)
     return nullptr;
-  auto path = new inbound_path(std::move(mgr), slots, std::move(sender));
+  auto path = new inbound_path(std::move(mgr), slots, std::move(sender), rtti);
   res.first->second.policy().handler.reset(path);
   return path;
 }
@@ -1100,14 +1132,13 @@ scheduled_actor::handle_open_stream_msg(mailbox_element& x) {
     auto sres = call_handler(default_handler_, this, x);
     switch (sres.flag) {
       default:
-        CAF_LOG_DEBUG("default handler was called for open_stream_msg:"
-                      << osm.msg);
+        CAF_LOG_DEBUG(
+          "default handler was called for open_stream_msg:" << osm.msg);
         fail(sec::stream_init_failed, "dropped open_stream_msg (no match)");
-        return im_dropped;
+        return invoke_message_result::dropped;
       case rt_skip:
-        CAF_LOG_DEBUG("default handler skipped open_stream_msg:"
-                      << osm.msg);
-        return im_skipped;
+        CAF_LOG_DEBUG("default handler skipped open_stream_msg:" << osm.msg);
+        return invoke_message_result::skipped;
     }
   };
   // Invoke behavior and dispatch on the result.
@@ -1120,25 +1151,11 @@ scheduled_actor::handle_open_stream_msg(mailbox_element& x) {
       CAF_LOG_DEBUG("no match in behavior, fall back to default handler");
       return fallback();
     case match_case::result::match: {
-      /*
-      if (f.ptr == nullptr) {
-        CAF_LOG_WARNING("actor did not return a stream manager after "
-                        "handling open_stream_msg");
-        fail(sec::stream_init_failed, "behavior did not create a manager");
-        return im_dropped;
-      }
-      stream_slots path_id{osm.slot, f.in_slot};
-      auto path = make_inbound_path(f.ptr, path_id, std::move(osm.prev_stage));
-      CAF_ASSERT(path != nullptr);
-      path->emit_ack_open(this, actor_cast<actor_addr>(osm.original_stage));
-      // Propagate handshake down the pipeline.
-      build_pipeline(f.in_slot, f.out_slot, std::move(f.ptr));
-      */
-      return im_success;
+      return invoke_message_result::consumed;
     }
     default:
       CAF_LOG_DEBUG("behavior skipped open_stream_msg:" << osm.msg);
-      return im_skipped; // nop
+      return invoke_message_result::skipped; // nop
   }
 }
 
@@ -1153,9 +1170,15 @@ scheduled_actor::advance_streams(actor_clock::time_point now) {
   auto bitmask = stream_ticks_.timeouts(now, {max_batch_delay_ticks_,
                                               credit_round_ticks_});
   // Force batches on all output paths.
-  if ((bitmask & 0x01) != 0) {
+  if ((bitmask & 0x01) != 0 && !stream_managers_.empty()) {
+    std::vector<stream_manager*> managers;
+    managers.reserve(stream_managers_.size());
     for (auto& kvp : stream_managers_)
-      kvp.second->out().force_emit_batches();
+      managers.emplace_back(kvp.second.get());
+    std::sort(managers.begin(), managers.end());
+    auto e = std::unique(managers.begin(), managers.end());
+    for (auto i = managers.begin(); i != e; ++i)
+      (*i)->out().force_emit_batches();
   }
   // Fill up credit on each input path.
   if ((bitmask & 0x02) != 0) {
@@ -1167,8 +1190,7 @@ scheduled_actor::advance_streams(actor_clock::time_point now) {
     for (auto& kvp : qs) {
       auto inptr = kvp.second.policy().handler.get();
       auto bs = static_cast<int32_t>(kvp.second.total_task_size());
-      inptr->emit_ack_batch(this, bs, inptr->mgr->out().max_capacity(),
-                            cycle, bc);
+      inptr->emit_ack_batch(this, bs, now, cycle, bc);
     }
   }
   return stream_ticks_.next_timeout(now, {max_batch_delay_ticks_,
