@@ -81,6 +81,13 @@
 #include "caf/detail/stream_stage_impl.hpp"
 #include "caf/detail/tick_emitter.hpp"
 
+CAF_BEGIN_TYPE_ID_BLOCK(native_streaming_classes, first_custom_type_id)
+
+  CAF_ADD_TYPE_ID(native_streaming_classes, (caf::stream<int32_t>) )
+  CAF_ADD_TYPE_ID(native_streaming_classes, (std::vector<int32_t>) )
+
+CAF_END_TYPE_ID_BLOCK(native_streaming_classes)
+
 using std::vector;
 
 using namespace caf;
@@ -228,16 +235,16 @@ public:
     return nullptr;
   }
 
-  void start_streaming(entity& ref, int num_messages) {
+  void start_streaming(entity& ref, int32_t num_messages) {
     CAF_REQUIRE_NOT_EQUAL(num_messages, 0);
-    using downstream_manager = broadcast_downstream_manager<int>;
+    using downstream_manager = broadcast_downstream_manager<int32_t>;
     struct driver final : public stream_source_driver<downstream_manager> {
     public:
-      driver(int sentinel) : x_(0), sentinel_(sentinel) {
+      driver(int32_t sentinel) : x_(0), sentinel_(sentinel) {
         // nop
       }
 
-      void pull(downstream<int>& out, size_t hint) override {
+      void pull(downstream<int32_t>& out, size_t hint) override {
         auto y = std::min(sentinel_, x_ + static_cast<int>(hint));
         while (x_ < y)
           out.push(x_++);
@@ -247,8 +254,8 @@ public:
         return x_ == sentinel_;
       }
     private:
-      int x_;
-      int sentinel_;
+      int32_t x_;
+      int32_t sentinel_;
     };
     auto mgr = detail::make_stream_source<driver>(this, num_messages);
     auto res = mgr->add_outbound_path(ref.ctrl());
@@ -260,9 +267,9 @@ public:
     using downstream_manager = broadcast_downstream_manager<int>;
     struct driver final : public stream_stage_driver<int, downstream_manager> {
     public:
-      using super = stream_stage_driver<int, downstream_manager>;
+      using super = stream_stage_driver<int32_t, downstream_manager>;
 
-      driver(downstream_manager& out, vector<int>* log)
+      driver(downstream_manager& out, vector<int32_t>* log)
         : super(out),
           log_(log) {
         // nop
@@ -342,7 +349,8 @@ public:
         for (auto& kvp : qs) {
           auto inptr = kvp.second.policy().handler.get();
           auto bs = static_cast<int32_t>(kvp.second.total_task_size());
-          inptr->emit_ack_batch(this, bs, 30, cycle, desired_batch_complexity);
+          inptr->emit_ack_batch(this, bs, now(), cycle,
+                                desired_batch_complexity);
         }
       }
     };
@@ -350,13 +358,15 @@ public:
   }
 
   inbound_path* make_inbound_path(stream_manager_ptr mgr, stream_slots slots,
-                                  strong_actor_ptr sender) override {
+                                  strong_actor_ptr sender,
+                                  rtti_pair rtti) override {
     using policy_type = policy::downstream_messages::nested;
     auto res = get<dmsg_id::value>(mbox.queues())
                .queues().emplace(slots.receiver, policy_type{nullptr});
     if (!res.second)
       return nullptr;
-    auto path = new inbound_path(std::move(mgr), slots, std::move(sender));
+    auto path = new inbound_path(std::move(mgr), slots, std::move(sender),
+                                 rtti);
     res.first->second.policy().handler.reset(path);
     return path;
   }
@@ -532,9 +542,17 @@ struct fixture {
     return *static_cast<entity*>(actor_cast<abstract_actor*>(hdl));
   }
 
+  static actor_system_config& init_config(actor_system_config& cfg) {
+    if (auto err = cfg.parse(caf::test::engine::argc(),
+                             caf::test::engine::argv()))
+      CAF_FAIL("parsing the config failed: " << to_string(err));
+    cfg.set("scheduler.policy", caf::atom("testing"));
+    cfg.add_message_types<id_block::native_streaming_classes>();
+    return cfg;
+  }
+
   fixture()
-      : sys(cfg.parse(caf::test::engine::argc(), caf::test::engine::argv())
-               .set("scheduler.policy", caf::atom("testing"))),
+      : sys(init_config(cfg)),
         sched(dynamic_cast<scheduler_type&>(sys.scheduler())),
         alice_hdl(spawn(sys, 0, "alice", tc)),
         bob_hdl(spawn(sys, 1, "bob", tc)),
@@ -606,7 +624,7 @@ vector<int> make_iota(int first, int last) {
   return result;
 }
 
-} // namespace <anonymous>
+} // namespace
 
 // -- unit tests ---------------------------------------------------------------
 
