@@ -21,51 +21,71 @@
 #define CAF_SUITE serialization
 #include "caf/test/unit_test.hpp"
 
-#include <new>
-#include <set>
-#include <list>
-#include <stack>
-#include <tuple>
-#include <locale>
-#include <memory>
-#include <string>
-#include <limits>
-#include <vector>
-#include <cstring>
-#include <sstream>
+#include <algorithm>
+#include <cassert>
 #include <cstdint>
 #include <cstring>
-#include <cassert>
-#include <iterator>
-#include <typeinfo>
-#include <iostream>
-#include <stdexcept>
-#include <algorithm>
 #include <functional>
+#include <iostream>
+#include <iterator>
+#include <limits>
+#include <list>
+#include <locale>
+#include <memory>
+#include <new>
+#include <set>
+#include <sstream>
+#include <stack>
+#include <stdexcept>
+#include <string>
+#include <tuple>
 #include <type_traits>
+#include <typeinfo>
+#include <vector>
 
-#include "caf/message.hpp"
-#include "caf/streambuf.hpp"
-#include "caf/serializer.hpp"
-#include "caf/ref_counted.hpp"
-#include "caf/deserializer.hpp"
 #include "caf/actor_system.hpp"
-#include "caf/proxy_registry.hpp"
-#include "caf/message_handler.hpp"
-#include "caf/event_based_actor.hpp"
-#include "caf/primitive_variant.hpp"
-#include "caf/binary_serializer.hpp"
-#include "caf/binary_deserializer.hpp"
 #include "caf/actor_system_config.hpp"
-#include "caf/make_type_erased_view.hpp"
+#include "caf/binary_deserializer.hpp"
+#include "caf/binary_serializer.hpp"
+#include "caf/deserializer.hpp"
+#include "caf/event_based_actor.hpp"
 #include "caf/make_type_erased_tuple_view.hpp"
+#include "caf/make_type_erased_view.hpp"
+#include "caf/message.hpp"
+#include "caf/message_handler.hpp"
+#include "caf/primitive_variant.hpp"
+#include "caf/proxy_registry.hpp"
+#include "caf/ref_counted.hpp"
+#include "caf/serializer.hpp"
+#include "caf/stream_deserializer.hpp"
+#include "caf/stream_serializer.hpp"
+#include "caf/streambuf.hpp"
+#include "caf/variant.hpp"
 
+#include "caf/detail/get_mac_addresses.hpp"
 #include "caf/detail/ieee_754.hpp"
 #include "caf/detail/int_list.hpp"
 #include "caf/detail/safe_equal.hpp"
 #include "caf/detail/type_traits.hpp"
-#include "caf/detail/enum_to_string.hpp"
-#include "caf/detail/get_mac_addresses.hpp"
+
+namespace {
+
+enum class test_enum : uint32_t;
+struct raw_struct;
+struct test_array;
+struct test_empty_non_pod;
+
+} // namespace
+
+CAF_BEGIN_TYPE_ID_BLOCK(serialization, first_custom_type_id)
+
+  CAF_ADD_TYPE_ID(serialization, (raw_struct))
+  CAF_ADD_TYPE_ID(serialization, (std::vector<bool>) )
+  CAF_ADD_TYPE_ID(serialization, (test_array))
+  CAF_ADD_TYPE_ID(serialization, (test_empty_non_pod))
+  CAF_ADD_TYPE_ID(serialization, (test_enum))
+
+CAF_END_TYPE_ID_BLOCK(serialization)
 
 using namespace std;
 using namespace caf;
@@ -91,13 +111,17 @@ bool operator==(const raw_struct& lhs, const raw_struct& rhs) {
 enum class test_enum : uint32_t {
   a,
   b,
-  c
+  c,
 };
 
-const char* test_enum_strings[] = { "a", "b", "c" };
+const char* test_enum_strings[] = {
+  "a",
+  "b",
+  "c",
+};
 
 std::string to_string(test_enum x) {
-  return detail::enum_to_string(x, test_enum_strings);
+  return test_enum_strings[static_cast<uint32_t>(x)];
 }
 
 struct test_array {
@@ -130,14 +154,12 @@ typename Inspector::result_type inspect(Inspector& f, test_empty_non_pod&) {
 class config : public actor_system_config {
 public:
   config() {
-    add_message_type<test_enum>("test_enum");
-    add_message_type<raw_struct>("raw_struct");
-    add_message_type<test_array>("test_array");
-    add_message_type<test_empty_non_pod>("test_empty_non_pod");
-    add_message_type<std::vector<bool>>("bool_vector");
+    puts("add_message_types");
+    add_message_types<id_block::serialization>();
   }
 };
 
+template <class Serializer, class Deserializer>
 struct fixture {
   int32_t i32 = -345;
   int64_t i64 = -1234567890123456789ll;
@@ -148,32 +170,33 @@ struct fixture {
   test_enum te = test_enum::b;
   string str = "Lorem ipsum dolor sit amet.";
   raw_struct rs;
-  test_array ta {
+  test_array ta{
     {0, 1, 2, 3},
-    {
-      {0, 1, 2, 3},
-      {4, 5, 6, 7}
-    },
+    {{0, 1, 2, 3}, {4, 5, 6, 7}},
   };
   int ra[3] = {1, 2, 3};
 
   config cfg;
   actor_system system;
-  scoped_execution_unit context;
   message msg;
+  message recursive;
 
   template <class T, class... Ts>
   vector<char> serialize(T& x, Ts&... xs) {
     vector<char> buf;
-    binary_serializer bs{&context, buf};
-    bs(x, xs...);
+    binary_serializer sink{system, buf};
+    if (auto err = sink(x, xs...))
+      CAF_FAIL("serialization failed: "
+               << system.render(err) << ", data: "
+               << deep_to_string(std::forward_as_tuple(x, xs...)));
     return buf;
   }
 
   template <class T, class... Ts>
   void deserialize(const vector<char>& buf, T& x, Ts&... xs) {
-    binary_deserializer bd{&context, buf};
-    bd(x, xs...);
+    binary_deserializer source{system, buf};
+    if (auto err = source(x, xs...))
+      CAF_FAIL("deserialization failed: " << system.render(err));
   }
 
   // serializes `x` and then deserializes and returns the serialized value
@@ -191,15 +214,20 @@ struct fixture {
     message result;
     auto tmp = make_message(x);
     deserialize(serialize(tmp), result);
-    CAF_REQUIRE(result.match_elements<T>());
+    if (!result.match_elements<T>())
+      CAF_FAIL("expected: " << x << ", got: " << result);
     return result.get_as<T>(0);
   }
 
-  fixture()
-      : system(cfg),
-        context(&system) {
+  fixture() : system(cfg) {
     rs.str.assign(string(str.rbegin(), str.rend()));
     msg = make_message(i32, i64, dur, ts, te, str, rs);
+    config_value::dictionary dict;
+    put(dict, "scheduler.policy", atom("none"));
+    put(dict, "scheduler.max-threads", 42);
+    put(dict, "nodes.preload",
+        make_config_value_list("sun", "venus", "mercury", "earth", "mars"));
+    recursive = make_message(config_value{std::move(dict)});
   }
 };
 
@@ -215,108 +243,139 @@ struct is_message {
     bool ok = false;
     // work around for gcc 4.8.4 bug
     auto tup = tie(v, vs...);
-    message_handler impl {
-      [&](T const& u, Ts const&... us) {
-        ok = tup == tie(u, us...);
-      }
+    message_handler impl{
+      [&](T const& u, Ts const&... us) { ok = tup == tie(u, us...); },
     };
     impl(msg);
     return ok;
   }
 };
 
-} // namespace <anonymous>
+} // namespace
 
-CAF_TEST_FIXTURE_SCOPE(serialization_tests, fixture)
+#define SERIALIZATION_TEST(name)                                               \
+  namespace {                                                                  \
+  template <class Serializer, class Deserializer>                              \
+  struct name##_tpl : fixture<Serializer, Deserializer> {                      \
+    using super = fixture<Serializer, Deserializer>;                           \
+    using super::i32;                                                          \
+    using super::i64;                                                          \
+    using super::f32;                                                          \
+    using super::f64;                                                          \
+    using super::dur;                                                          \
+    using super::ts;                                                           \
+    using super::te;                                                           \
+    using super::str;                                                          \
+    using super::rs;                                                           \
+    using super::ta;                                                           \
+    using super::ra;                                                           \
+    using super::system;                                                       \
+    using super::msg;                                                          \
+    using super::recursive;                                                    \
+    using super::serialize;                                                    \
+    using super::deserialize;                                                  \
+    using super::roundtrip;                                                    \
+    using super::msg_roundtrip;                                                \
+    void run_test_impl();                                                      \
+  };                                                                           \
+  using name##_binary = name##_tpl<binary_serializer, binary_deserializer>;    \
+  using name##_stream                                                          \
+    = name##_tpl<stream_serializer<vectorbuf>, stream_deserializer<charbuf>>;  \
+  ::caf::test::detail::adder<::caf::test::test_impl<name##_binary>>            \
+    CAF_UNIQUE(a_binary){CAF_XSTR(CAF_SUITE), CAF_XSTR(name##_binary), false}; \
+  ::caf::test::detail::adder<::caf::test::test_impl<name##_stream>>            \
+    CAF_UNIQUE(a_stream){CAF_XSTR(CAF_SUITE), CAF_XSTR(name##_stream), false}; \
+  }                                                                            \
+  template <class Serializer, class Deserializer>                              \
+  void name##_tpl<Serializer, Deserializer>::run_test_impl()
 
-CAF_TEST(ieee_754_conversion) {
+SERIALIZATION_TEST(ieee_754_conversion) {
   // check conversion of float
-  float f1 = 3.1415925f;         // float value
+  float f1 = 3.1415925f;              // float value
   auto p1 = caf::detail::pack754(f1); // packet value
   CAF_CHECK_EQUAL(p1, static_cast<decltype(p1)>(0x40490FDA));
   auto u1 = caf::detail::unpack754(p1); // unpacked value
   CAF_CHECK_EQUAL(f1, u1);
   // check conversion of double
-  double f2 = 3.14159265358979311600;  // double value
+  double f2 = 3.14159265358979311600; // double value
   auto p2 = caf::detail::pack754(f2); // packet value
   CAF_CHECK_EQUAL(p2, static_cast<decltype(p2)>(0x400921FB54442D18));
   auto u2 = caf::detail::unpack754(p2); // unpacked value
   CAF_CHECK_EQUAL(f2, u2);
 }
 
-CAF_TEST(i32_values) {
+SERIALIZATION_TEST(i32_values) {
   auto buf = serialize(i32);
   int32_t x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(i32, x);
 }
 
-CAF_TEST(i64_values) {
+SERIALIZATION_TEST(i64_values) {
   auto buf = serialize(i64);
   int64_t x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(i64, x);
 }
 
-CAF_TEST(float_values) {
+SERIALIZATION_TEST(float_values) {
   auto buf = serialize(f32);
   float x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(f32, x);
 }
 
-CAF_TEST(double_values) {
+SERIALIZATION_TEST(double_values) {
   auto buf = serialize(f64);
   double x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(f64, x);
 }
 
-CAF_TEST(duration_values) {
+SERIALIZATION_TEST(duration_values) {
   auto buf = serialize(dur);
   duration x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(dur, x);
 }
 
-CAF_TEST(timestamp_values) {
+SERIALIZATION_TEST(timestamp_values) {
   auto buf = serialize(ts);
   timestamp x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(ts, x);
 }
 
-CAF_TEST(enum_classes) {
+SERIALIZATION_TEST(enum_classes) {
   auto buf = serialize(te);
   test_enum x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(te, x);
 }
 
-CAF_TEST(strings) {
+SERIALIZATION_TEST(strings) {
   auto buf = serialize(str);
   string x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(str, x);
 }
 
-CAF_TEST(custom_struct) {
+SERIALIZATION_TEST(custom_struct) {
   auto buf = serialize(rs);
   raw_struct x;
   deserialize(buf, x);
   CAF_CHECK_EQUAL(rs, x);
 }
 
-CAF_TEST(atoms) {
+SERIALIZATION_TEST(atoms) {
   auto foo = atom("foo");
   CAF_CHECK_EQUAL(foo, roundtrip(foo));
   CAF_CHECK_EQUAL(foo, msg_roundtrip(foo));
-  using bar_atom = atom_constant<atom("bar")>;
-  CAF_CHECK_EQUAL(bar_atom::value, roundtrip(atom("bar")));
-  CAF_CHECK_EQUAL(bar_atom::value, msg_roundtrip(atom("bar")));
+  CAF_CHECK_EQUAL(atom("bar"), roundtrip(atom("bar")));
+  CAF_CHECK_EQUAL(atom("bar"), msg_roundtrip(atom("bar")));
 }
 
-CAF_TEST(raw_arrays) {
+SERIALIZATION_TEST(raw_arrays) {
   auto buf = serialize(ra);
   int x[3];
   deserialize(buf, x);
@@ -324,7 +383,7 @@ CAF_TEST(raw_arrays) {
     CAF_CHECK_EQUAL(ra[i], x[i]);
 }
 
-CAF_TEST(arrays) {
+SERIALIZATION_TEST(arrays) {
   auto buf = serialize(ta);
   test_array x;
   deserialize(buf, x);
@@ -335,7 +394,7 @@ CAF_TEST(arrays) {
       CAF_CHECK_EQUAL(ta.value2[i][j], x.value2[i][j]);
 }
 
-CAF_TEST(empty_non_pods) {
+SERIALIZATION_TEST(empty_non_pods) {
   test_empty_non_pod x;
   auto buf = serialize(x);
   CAF_REQUIRE(buf.empty());
@@ -354,7 +413,7 @@ std::string hexstr(const std::vector<char>& buf) {
   return oss.str();
 }
 
-CAF_TEST(messages) {
+SERIALIZATION_TEST(messages) {
   // serialize original message which uses tuple_vals internally and
   // deserialize into a message which uses type_erased_value pointers
   message x;
@@ -369,9 +428,10 @@ CAF_TEST(messages) {
   deserialize(buf2, y);
   CAF_CHECK_EQUAL(to_string(msg), to_string(y));
   CAF_CHECK(is_message(y).equal(i32, i64, dur, ts, te, str, rs));
+  CAF_CHECK_EQUAL(to_string(recursive), to_string(roundtrip(recursive)));
 }
 
-CAF_TEST(multiple_messages) {
+SERIALIZATION_TEST(multiple_messages) {
   auto m = make_message(rs, te);
   auto buf = serialize(te, m, msg);
   test_enum t;
@@ -384,16 +444,15 @@ CAF_TEST(multiple_messages) {
   CAF_CHECK(is_message(m2).equal(i32, i64, dur, ts, te, str, rs));
 }
 
-
-CAF_TEST(type_erased_value) {
+SERIALIZATION_TEST(type_erased_value) {
   auto buf = serialize(str);
   type_erased_value_ptr ptr{new type_erased_value_impl<std::string>};
-  binary_deserializer bd{&context, buf.data(), buf.size()};
-  ptr->load(bd);
+  binary_deserializer source{system, buf};
+  ptr->load(source);
   CAF_CHECK_EQUAL(str, *reinterpret_cast<const std::string*>(ptr->get()));
 }
 
-CAF_TEST(type_erased_view) {
+SERIALIZATION_TEST(type_erased_view) {
   auto str_view = make_type_erased_view(str);
   auto buf = serialize(str_view);
   std::string res;
@@ -401,7 +460,7 @@ CAF_TEST(type_erased_view) {
   CAF_CHECK_EQUAL(str, res);
 }
 
-CAF_TEST(type_erased_tuple) {
+SERIALIZATION_TEST(type_erased_tuple) {
   auto tview = make_type_erased_tuple_view(str, i32);
   CAF_CHECK_EQUAL(to_string(tview), deep_to_string(std::make_tuple(str, i32)));
   auto buf = serialize(tview);
@@ -415,7 +474,7 @@ CAF_TEST(type_erased_tuple) {
   CAF_CHECK_EQUAL(to_string(tview), deep_to_string(std::make_tuple(str, i32)));
 }
 
-CAF_TEST(streambuf_serialization) {
+SERIALIZATION_TEST(streambuf_serialization) {
   auto data = std::string{"The quick brown fox jumps over the lazy dog"};
   std::vector<char> buf;
   // First, we check the standard use case in CAF where stream serializers own
@@ -443,7 +502,7 @@ CAF_TEST(streambuf_serialization) {
   CAF_CHECK(data == target);
 }
 
-CAF_TEST(byte_sequence_optimization) {
+SERIALIZATION_TEST(byte_sequence_optimization) {
   std::vector<uint8_t> data(42);
   std::fill(data.begin(), data.end(), 0x2a);
   std::vector<uint8_t> buf;
@@ -458,11 +517,11 @@ CAF_TEST(byte_sequence_optimization) {
   e = bd(data);
   CAF_REQUIRE(!e);
   CAF_CHECK_EQUAL(data.size(), 42u);
-  CAF_CHECK(std::all_of(data.begin(), data.end(),
-                        [](uint8_t c) { return c == 0x2a; }));
+  CAF_CHECK(
+    std::all_of(data.begin(), data.end(), [](uint8_t c) { return c == 0x2a; }));
 }
 
-CAF_TEST(long_sequences) {
+SERIALIZATION_TEST(long_sequences) {
   std::vector<char> data;
   binary_serializer sink{nullptr, data};
   size_t n = std::numeric_limits<uint32_t>::max();
@@ -475,7 +534,7 @@ CAF_TEST(long_sequences) {
   CAF_CHECK_EQUAL(n, m);
 }
 
-CAF_TEST(non-empty vector) {
+SERIALIZATION_TEST(non_empty_vector) {
   CAF_MESSAGE("deserializing into a non-empty vector overrides any content");
   std::vector<int> foo{1, 2, 3};
   std::vector<int> bar{0};
@@ -484,60 +543,79 @@ CAF_TEST(non-empty vector) {
   CAF_CHECK_EQUAL(foo, bar);
 }
 
+SERIALIZATION_TEST(variant_with_tree_types) {
+  CAF_MESSAGE("deserializing into a non-empty vector overrides any content");
+  using test_variant = variant<int, double, std::string>;
+  test_variant x{42};
+  CAF_CHECK_EQUAL(x, roundtrip(x));
+  x = 12.34;
+  CAF_CHECK_EQUAL(x, roundtrip(x));
+  x = std::string{"foobar"};
+  CAF_CHECK_EQUAL(x, roundtrip(x));
+}
+
 // -- our vector<bool> serialization packs into an uint64_t. Hence, the
 // critical sizes to test are 0, 1, 63, 64, and 65.
 
-CAF_TEST(bool_vector_size_0) {
+SERIALIZATION_TEST(bool_vector_size_0) {
   std::vector<bool> xs;
   CAF_CHECK_EQUAL(deep_to_string(xs), "[]");
   CAF_CHECK_EQUAL(xs, roundtrip(xs));
   CAF_CHECK_EQUAL(xs, msg_roundtrip(xs));
 }
 
-CAF_TEST(bool_vector_size_1) {
+SERIALIZATION_TEST(bool_vector_size_1) {
   std::vector<bool> xs{true};
-  CAF_CHECK_EQUAL(deep_to_string(xs), "[1]");
+  CAF_CHECK_EQUAL(deep_to_string(xs), "[true]");
   CAF_CHECK_EQUAL(xs, roundtrip(xs));
   CAF_CHECK_EQUAL(xs, msg_roundtrip(xs));
 }
 
-CAF_TEST(bool_vector_size_63) {
+SERIALIZATION_TEST(bool_vector_size_63) {
   std::vector<bool> xs;
   for (int i = 0; i < 63; ++i)
     xs.push_back(i % 3 == 0);
-  CAF_CHECK_EQUAL(deep_to_string(xs),
-                  "[1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, "
-                  "0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, "
-                  "1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, "
-                  "0, 1, 0, 0]");
+  CAF_CHECK_EQUAL(
+    deep_to_string(xs),
+    "[true, false, false, true, false, false, true, false, false, true, false, "
+    "false, true, false, false, true, false, false, true, false, false, true, "
+    "false, false, true, false, false, true, false, false, true, false, false, "
+    "true, false, false, true, false, false, true, false, false, true, false, "
+    "false, true, false, false, true, false, false, true, false, false, true, "
+    "false, false, true, false, false, true, false, false]");
   CAF_CHECK_EQUAL(xs, roundtrip(xs));
   CAF_CHECK_EQUAL(xs, msg_roundtrip(xs));
 }
 
-CAF_TEST(bool_vector_size_64) {
+SERIALIZATION_TEST(bool_vector_size_64) {
   std::vector<bool> xs;
   for (int i = 0; i < 64; ++i)
     xs.push_back(i % 5 == 0);
   CAF_CHECK_EQUAL(deep_to_string(xs),
-                  "[1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, "
-                  "0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, "
-                  "0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, "
-                  "0, 1, 0, 0, 0]");
+                  "[true, false, false, false, false, true, false, false, "
+                  "false, false, true, false, false, false, false, true, "
+                  "false, false, false, false, true, false, false, false, "
+                  "false, true, false, false, false, false, true, false, "
+                  "false, false, false, true, false, false, false, false, "
+                  "true, false, false, false, false, true, false, false, "
+                  "false, false, true, false, false, false, false, true, "
+                  "false, false, false, false, true, false, false, false]");
   CAF_CHECK_EQUAL(xs, roundtrip(xs));
   CAF_CHECK_EQUAL(xs, msg_roundtrip(xs));
 }
 
-CAF_TEST(bool_vector_size_65) {
+SERIALIZATION_TEST(bool_vector_size_65) {
   std::vector<bool> xs;
   for (int i = 0; i < 65; ++i)
     xs.push_back(!(i % 7 == 0));
-  CAF_CHECK_EQUAL(deep_to_string(xs),
-                  "[0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, "
-                  "1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, "
-                  "1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, "
-                  "1, 1, 1, 1, 0, 1]");
+  CAF_CHECK_EQUAL(
+    deep_to_string(xs),
+    "[false, true, true, true, true, true, true, false, true, true, true, "
+    "true, true, true, false, true, true, true, true, true, true, false, true, "
+    "true, true, true, true, true, false, true, true, true, true, true, true, "
+    "false, true, true, true, true, true, true, false, true, true, true, true, "
+    "true, true, false, true, true, true, true, true, true, false, true, true, "
+    "true, true, true, true, false, true]");
   CAF_CHECK_EQUAL(xs, roundtrip(xs));
   CAF_CHECK_EQUAL(xs, msg_roundtrip(xs));
 }
-
-CAF_TEST_FIXTURE_SCOPE_END()
